# Licensed to the Apache Software Foundation (ASF) under one or more
# contributor license agreements.  See the NOTICE file distributed with
# this work for additional information regarding copyright ownership.
# The ASF licenses this file to You under the Apache License, Version 2.0
# (the "License"); you may not use this file except in compliance with
# the License.  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from libcloud.dns.types import Provider
from libcloud.dns.types import OLD_CONSTANT_TO_NEW_MAPPING
from libcloud.common.providers import get_driver as _get_provider_driver
from libcloud.common.providers import set_driver as _set_provider_driver

__all__ = [
    'DRIVERS',

    'get_driver',
    'set_driver'
]

DRIVERS = {
    Provider.DUMMY:
    ('libcloud.dns.drivers.dummy', 'DummyDNSDriver'),
    Provider.LINODE:
    ('libcloud.dns.drivers.linode', 'LinodeDNSDriver'),
    Provider.ZERIGO:
    ('libcloud.dns.drivers.zerigo', 'ZerigoDNSDriver'),
    Provider.RACKSPACE:
    ('libcloud.dns.drivers.rackspace', 'RackspaceDNSDriver'),
    Provider.HOSTVIRTUAL:
    ('libcloud.dns.drivers.hostvirtual', 'HostVirtualDNSDriver'),
    Provider.ROUTE53:
    ('libcloud.dns.drivers.route53', 'Route53DNSDriver'),
    Provider.GANDI:
    ('libcloud.dns.drivers.gandi', 'GandiDNSDriver'),
    Provider.GOOGLE: ('libcloud.dns.drivers.google', 'GoogleDNSDriver'),
    Provider.SOFTLAYER:
    ('libcloud.dns.drivers.softlayer', 'SoftLayerDNSDriver'),
    Provider.DIGITAL_OCEAN:
    ('libcloud.dns.drivers.digitalocean', 'DigitalOceanDNSDriver'),
    Provider.WORLDWIDEDNS:
    ('libcloud.dns.drivers.worldwidedns', 'WorldWideDNSDriver'),
    Provider.DNSIMPLE:
    ('libcloud.dns.drivers.dnsimple', 'DNSimpleDNSDriver'),
    Provider.POINTDNS:
    ('libcloud.dns.drivers.pointdns', 'PointDNSDriver'),
    Provider.VULTR:
    ('libcloud.dns.drivers.vultr', 'VultrDNSDriver'),
    Provider.LIQUIDWEB:
    ('libcloud.dns.drivers.liquidweb', 'LiquidWebDNSDriver'),
    Provider.ZONOMI:
    ('libcloud.dns.drivers.zonomi', 'ZonomiDNSDriver'),
    Provider.DURABLEDNS:
    ('libcloud.dns.drivers.durabledns', 'DurableDNSDriver'),
    Provider.AURORADNS:
    ('libcloud.dns.drivers.auroradns', 'AuroraDNSDriver'),
    Provider.GODADDY:
    ('libcloud.dns.drivers.godaddy', 'GoDaddyDNSDriver'),
    Provider.CLOUDFLARE:
    ('libcloud.dns.drivers.cloudflare', 'CloudFlareDNSDriver'),
    Provider.NFSN:
    ('libcloud.dns.drivers.nfsn', 'NFSNDNSDriver'),
    Provider.NSONE:
    ('libcloud.dns.drivers.nsone', 'NsOneDNSDriver'),
    Provider.LUADNS:
    ('libcloud.dns.drivers.luadns', 'LuadnsDNSDriver'),
    Provider.BUDDYNS:
    ('libcloud.dns.drivers.buddyns', 'BuddyNSDNSDriver'),
    Provider.POWERDNS:
    ('libcloud.dns.drivers.powerdns', 'PowerDNSDriver'),

    # Deprecated
    Provider.RACKSPACE_US:
    ('libcloud.dns.drivers.rackspace', 'RackspaceUSDNSDriver'),
    Provider.RACKSPACE_UK:
    ('libcloud.dns.drivers.rackspace', 'RackspaceUKDNSDriver')
}


def get_driver(provider):
    deprecated_constants = OLD_CONSTANT_TO_NEW_MAPPING
    return _get_provider_driver(drivers=DRIVERS, provider=provider,
                                deprecated_constants=deprecated_constants)


def set_driver(provider, module, klass):
    return _set_provider_driver(drivers=DRIVERS, provider=provider,
                                module=module, klass=klass)



# -*- coding: utf-8 -*-
# Licensed to the Apache Software Foundation (ASF) under one or more
# contributor license agreements.  See the NOTICE file distributed with
# this work for additional information regarding copyright ownership.
# The ASF licenses this file to You under the Apache License, Version 2.0
# (the "License"); you may not use this file except in compliance with
# the License.  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from __future__ import unicode_literals

import sys
import unittest

from libcloud.common.types import LibcloudError
from libcloud.compute.base import Node, NodeAuthPassword, NodeImage, \
    NodeLocation, NodeSize, StorageVolume, VolumeSnapshot
from libcloud.compute.drivers.ecs import ECSDriver
from libcloud.compute.types import NodeState, StorageVolumeState
from libcloud.test import MockHttpTestCase, LibcloudTestCase
from libcloud.test.file_fixtures import ComputeFileFixtures
from libcloud.test.secrets import ECS_PARAMS
from libcloud.utils.py3 import httplib


class ECSDriverTestCase(LibcloudTestCase):
    region = 'cn-qingdao'
    zone = 'cn-qingdao-b'
    image_id = 'ubuntu1404_64_20G_aliaegis_20150325.vhd'

    def setUp(self):
        ECSMockHttp.test = self
        ECSDriver.connectionCls.conn_classes = (ECSMockHttp, ECSMockHttp)
        ECSMockHttp.use_param = 'Action'
        ECSMockHttp.type = None

        self.driver = ECSDriver(*ECS_PARAMS, region=self.region)
        self.fake_size = NodeSize('ecs.t1.small', 'ecs t1 small',
                                  None, None, None, None,
                                  self.driver)
        self.fake_image = NodeImage(self.image_id, name='ubuntu 14.04 64bit',
                                    driver=self.driver)
        self.fake_node = Node(id='fake-node1', name='fake-node',
                              state=NodeState.RUNNING,
                              public_ips=None,
                              private_ips=None,
                              driver=self.driver)
        self.fake_volume = StorageVolume(id='fake-volume1', name='fake-volume',
                                         size=self.fake_size,
                                         driver=self.driver)
        self.fake_snapshot = VolumeSnapshot(id='fake-snapshot1',
                                            driver=self.driver)
        self.fake_location = NodeLocation(id=self.region, name=self.region,
                                          country=None, driver=self.driver)

    def test_list_nodes(self):
        nodes = self.driver.list_nodes()
        self.assertIsNotNone(nodes)
        self.assertEqual(1, len(nodes))
        node = nodes[0]
        self.assertEqual('iZ28n7dkvovZ', node.name)
        self.assertEqual('i-28n7dkvov', node.id)
        self.assertEqual(NodeState.PENDING, node.state)
        self.assertEqual(1, len(node.public_ips))
        self.assertEqual('114.215.124.73', node.public_ips[0])
        self.assertEqual(1, len(node.private_ips))
        self.assertEqual('10.163.197.74', node.private_ips[0])
        expected_extra = {
            'image_id': 'ubuntu1404_64_20G_aliaegis_20150325.vhd',
            'description': '',
            'instance_type_family': 'ecs.t1',
            'zone_id': 'cn-qingdao-b',
            'internet_charge_type': 'PayByTraffic',
            'serial_number': 'ca0122d9-374d-4fce-9fc0-71f7c3eaf1c3',
            'io_optimized': 'false',
            'device_available': 'true',
            'instance_network_type': 'classic',
            'hostname': 'iZ28n7dkvovZ',
            'instance_type': 'ecs.t1.small',
            'creation_time': '2015-12-27T07:35Z',
            'instance_charge_type': 'PostPaid',
            'expired_time': '2999-09-08T16:00Z'
        }
        self._validate_extras(expected_extra, node.extra)
        vpc = {
            'vpc_id': '',
            'vswitch_id': '',
            'private_ip_address': None,
            'nat_ip_address': ''
        }
        self._validate_extras(vpc, node.extra['vpc_attributes'])
        eip_address = {
            'allocation_id': '',
            'ip_address': '',
            'internet_charge_type': '',
            'bandwidth': None
        }
        self._validate_extras(eip_address, node.extra['eip_address'])
        self.assertIsNone(node.extra['operation_locks']['lock_reason'])

    def test_list_nodes_with_ex_node_ids(self):
        ECSMockHttp.type = 'list_nodes_ex_node_ids'
        nodes = self.driver.list_nodes(ex_node_ids=['i-28n7dkvov',
                                                    'not-existed-id'])
        self.assertIsNotNone(nodes)

    def test_list_nodes_with_ex_filters(self):
        ECSMockHttp.type = 'list_nodes_ex_filters'
        nodes = self.driver.list_nodes(ex_filters={'ZoneId': self.zone})
        self.assertIsNotNone(nodes)

    def _validate_extras(self, expected, actual):
        self.assertIsNotNone(actual)
        for key, value in iter(expected.items()):
            self.assertTrue(key in actual)
            self.assertEqual(value, actual[key], ('extra %(key)s not equal, '
                                                  'expected: "%(expected)s", '
                                                  'actual: "%(actual)s"' %
                                                  {'key': key,
                                                   'expected': value,
                                                   'actual': actual[key]}))

    def test_create_node(self):
        ECSMockHttp.type = 'create_node'
        name = 'test_create_node'
        node = self.driver.create_node(name=name, image=self.fake_image,
                                       size=self.fake_size,
                                       ex_security_group_id='sg-28ou0f3xa',
                                       ex_description='description',
                                       ex_internet_charge_type='PayByTraffic',
                                       ex_internet_max_bandwidth_out=1,
                                       ex_internet_max_bandwidth_in=200,
                                       ex_hostname='hostname',
                                       auth=NodeAuthPassword('password'),
                                       ex_io_optimized=True,
                                       ex_system_disk={'category': 'cloud',
                                                       'disk_name': 'root',
                                                       'description': 'sys'},
                                       ex_vswitch_id='vswitch-id1',
                                       ex_private_ip_address='1.1.1.2',
                                       ex_client_token='client_token')
        self.assertIsNotNone(node)

    def test_create_node_with_data_disk(self):
        ECSMockHttp.type = 'create_node_with_data'
        self.name = 'test_create_node'
        self.data_disk = {
            'size': 5,
            'category': self.driver.disk_categories.CLOUD,
            'disk_name': 'data1',
            'description': 'description',
            'device': '/dev/xvdb',
            'delete_with_instance': True}
        node = self.driver.create_node(name=self.name, image=self.fake_image,
                                       size=self.fake_size,
                                       ex_security_group_id='sg-28ou0f3xa',
                                       ex_data_disks=self.data_disk)
        self.assertIsNotNone(node)

    def test_list_sizes(self):
        sizes = self.driver.list_sizes()
        self.assertEqual(2, len(sizes))
        size = sizes[0]
        self.assertEqual('ecs.t1.xsmall', size.id)
        self.assertEqual('ecs.t1.xsmall', size.name)
        self.assertEqual(0.5, size.ram)
        self.assertEqual(1, size.extra['cpu_core_count'])
        self.assertEqual('ecs.t1', size.extra['instance_type_family'])
        size = sizes[1]
        self.assertEqual('ecs.s2.small', size.id)
        self.assertEqual('ecs.s2.small', size.name)
        self.assertEqual(1.0, size.ram)
        self.assertEqual(2, size.extra['cpu_core_count'])
        self.assertEqual('ecs.s2', size.extra['instance_type_family'])

    def test_list_locations(self):
        locations = self.driver.list_locations()
        self.assertEqual(9, len(locations))
        location = locations[0]
        self.assertEqual('ap-southeast-1', location.id)
        self.assertEqual('亚太（新加坡）', location.name)
        self.assertIsNone(location.country)

    def test_create_node_without_sg_id_exception(self):
        name = 'test_create_node_without_sg_id_exception'
        self.assertRaises(AttributeError, self.driver.create_node,
                          name=name, image=self.fake_image,
                          size=self.fake_size)

    def test_creat_node_paybytraffic_exception(self):
        name = 'test_create_node_paybytraffic_exception'
        self.assertRaises(AttributeError, self.driver.create_node,
                          name=name, image=self.fake_image,
                          size=self.fake_size,
                          ex_security_group_id='sg-id1',
                          ex_internet_charge_type='PayByTraffic')

    def test_create_node_ex_system_disk_exception(self):
        name = 'test_creat_node_ex_system_disk_exception'
        self.assertRaises(AttributeError, self.driver.create_node,
                          name=name, image=self.fake_image,
                          size=self.fake_size,
                          ex_security_group_id='sg-id1',
                          ex_system_disk=None)

    def test_create_node_ex_private_ip_address_exception(self):
        name = 'test_create_node_ex_private_ip_address_exception'
        self.assertRaises(AttributeError, self.driver.create_node,
                          name=name, image=self.fake_image,
                          size=self.fake_size,
                          ex_security_group_id='sg-id1',
                          ex_private_ip_address='1.1.1.2')

    def test_reboot_node(self):
        ECSMockHttp.type = 'reboot_node'
        result = self.driver.reboot_node(self.fake_node)
        self.assertTrue(result)

    def test_reboot_node_with_ex_force_stop(self):
        ECSMockHttp.type = 'reboot_node_force_stop'
        result = self.driver.reboot_node(self.fake_node, ex_force_stop=True)
        self.assertTrue(result)

    def test_destroy_node(self):
        ECSMockHttp.type = 'destroy_node'
        result = self.driver.destroy_node(self.fake_node)
        self.assertTrue(result)

    def test_ex_start_node(self):
        ECSMockHttp.type = 'start_node'
        result = self.driver.ex_start_node(self.fake_node)
        self.assertTrue(result)

    def test_ex_stop_node(self):
        ECSMockHttp.type = 'stop_node'
        result = self.driver.ex_stop_node(self.fake_node)
        self.assertTrue(result)

    def test_stop_node_with_ex_force_stop(self):
        ECSMockHttp.type = 'stop_node_force_stop'
        result = self.driver.ex_stop_node(self.fake_node, ex_force_stop=True)
        self.assertTrue(result)

    def test_list_volumes(self):
        volumes = self.driver.list_volumes()
        self.assertEqual(2, len(volumes))
        volume = volumes[0]
        self.assertEqual('d-28m5zbua0', volume.id)
        self.assertEqual('', volume.name)
        self.assertEqual(5, volume.size)
        self.assertEqual(StorageVolumeState.AVAILABLE, volume.state)
        expected_extras = {
            'region_id': 'cn-qingdao',
            'zone_id': 'cn-qingdao-b',
            'description': '',
            'type': 'data',
            'category': 'cloud',
            'image_id': '',
            'source_snapshot_id': '',
            'product_code': '',
            'portable': True,
            'instance_id': '',
            'device': '',
            'delete_with_instance': False,
            'enable_auto_snapshot': False,
            'creation_time': '2014-07-23T02:44:07Z',
            'attached_time': '2014-07-23T07:47:35Z',
            'detached_time': '2014-07-23T08:28:48Z',
            'disk_charge_type': 'PostPaid',
            'operation_locks': {'lock_reason': None}
        }
        self._validate_extras(expected_extras, volume.extra)
        volume = volumes[1]
        self.assertEqual('d-28zfrmo13', volume.id)
        self.assertEqual('ubuntu1404sys', volume.name)
        self.assertEqual(5, volume.size)
        self.assertEqual(StorageVolumeState.INUSE, volume.state)
        expected_extras = {
            'region_id': 'cn-qingdao',
            'zone_id': 'cn-qingdao-b',
            'description': 'Description',
            'type': 'system',
            'category': 'cloud',
            'image_id': 'ubuntu1404_64_20G_aliaegis_20150325.vhd',
            'source_snapshot_id': '',
            'product_code': '',
            'portable': False,
            'instance_id': 'i-28whl2nj2',
            'device': '/dev/xvda',
            'delete_with_instance': True,
            'enable_auto_snapshot': True,
            'creation_time': '2014-07-23T02:44:06Z',
            'attached_time': '2016-01-04T15:02:17Z',
            'detached_time': '',
            'disk_charge_type': 'PostPaid',
            'operation_locks': {'lock_reason': None}
        }
        self._validate_extras(expected_extras, volume.extra)

    def test_list_volumes_with_ex_volume_ids(self):
        ECSMockHttp.type = 'list_volumes_ex_volume_ids'
        volumes = self.driver.list_volumes(ex_volume_ids=['i-28n7dkvov',
                                                          'not-existed-id'])
        self.assertIsNotNone(volumes)

    def test_list_volumes_with_ex_filters(self):
        ECSMockHttp.type = 'list_volumes_ex_filters'
        ex_filters = {'InstanceId': self.fake_node.id}
        volumes = self.driver.list_volumes(ex_filters=ex_filters)
        self.assertIsNotNone(volumes)

    def test_list_volume_snapshots(self):
        snapshots = self.driver.list_volume_snapshots(self.fake_volume)
        self.assertEqual(1, len(snapshots))

    def test_list_volume_snapshots_with_ex_snapshot_ids(self):
        ECSMockHttp.type = 'list_volume_snapshots_ex_snapshot_ids'
        ex_snapshot_ids = ['fake-snapshot1']
        self.driver.list_volume_snapshots(self.fake_volume,
                                          ex_snapshot_ids=ex_snapshot_ids)

    def test_list_volume_snapshots_with_ex_filters(self):
        ECSMockHttp.type = 'list_volume_snapshots_ex_filters'
        ex_filters = {'InstanceId': self.fake_node.id}
        self.driver.list_volume_snapshots(self.fake_volume,
                                          ex_filters=ex_filters)

    def test_create_volume(self):
        ECSMockHttp.type = 'create_volume'
        self.volume_size = 1
        self.volume_name = 'fake-volume-name'
        self.description = 'fake-description'
        self.disk_category = 'system'
        self.client_token = 'client_token'
        volume = self.driver.create_volume(self.volume_size, self.volume_name,
                                           snapshot=self.fake_snapshot,
                                           ex_zone_id=self.zone,
                                           ex_description=self.description,
                                           ex_disk_category=self.disk_category,
                                           ex_client_token=self.client_token)
        self.assertIsNotNone(volume)

    def test_create_volume_without_ex_zone_id_exception(self):
        self.assertRaises(AttributeError,
                          self.driver.create_volume,
                          1, 'fake-volume-name')

    def test_create_volume_snapshot(self):
        ECSMockHttp.type = 'create_volume_snapshot'
        self.snapshot_name = 'fake-snapshot1'
        self.description = 'fake-description'
        self.client_token = 'client-token'
        snapshot = self.driver.create_volume_snapshot(
            self.fake_volume, name=self.snapshot_name,
            ex_description=self.description,
            ex_client_token=self.client_token)
        self.assertIsNotNone(snapshot)

    def test_attach_volume(self):
        self.device = '/dev/sdb'
        self.delete_with_instance = True
        attached = self.driver.attach_volume(
            self.fake_node, self.fake_volume, device=self.device,
            ex_delete_with_instance=self.delete_with_instance)
        self.assertTrue(attached)

    def test_detach_volume(self):
        self.instance_id = 'fake-node1'
        result = self.driver.detach_volume(self.fake_volume,
                                           ex_instance_id=self.instance_id)
        self.assertTrue(result)

    def test_detach_volume_query_instance_id(self):
        ECSMockHttp.type = 'detach_volume'
        result = self.driver.detach_volume(self.fake_volume)
        self.assertTrue(result)

    def test_detach_volume_query_instance_id_exception(self):
        self.assertRaises(AttributeError, self.driver.detach_volume,
                          self.fake_volume)

    def test_destroy_volume(self):
        ECSMockHttp.type = 'destroy_volume'
        result = self.driver.destroy_volume(self.fake_volume)
        self.assertTrue(result)

    def test_destroy_volume_query_volumes_exception(self):
        self.assertRaises(LibcloudError, self.driver.destroy_volume,
                          self.fake_volume)

    def test_destroy_volume_state_exception(self):
        ECSMockHttp.type = 'destroy_volume_state'
        self.assertRaises(LibcloudError, self.driver.destroy_volume,
                          self.fake_volume)

    def test_destroy_volume_snapshot(self):
        result = self.driver.destroy_volume_snapshot(self.fake_snapshot)
        self.assertTrue(result)

    def test_destroy_volume_snapshot_exception(self):
        self.assertRaises(AttributeError, self.driver.destroy_volume_snapshot,
                          self.fake_volume)

    def test_list_images(self):
        images = self.driver.list_images(self.fake_location)
        self.assertEqual(1, len(images))
        image = images[0]
        self.assertEqual('freebsd1001_64_20G_aliaegis_20150527.vhd', image.id)
        self.assertEqual('freebsd1001_64_20G_aliaegis_20150527.vhd',
                         image.name)
        expected_extra = {
            'image_version': '1.0.0',
            'os_type': 'linux',
            'platform': 'Freebsd',
            'architecture': 'x86_64',
            'description': 'freebsd1001_64_20G_aliaegis_20150527.vhd',
            'size': 20,
            'image_owner_alias': 'system',
            'os_name': 'FreeBSD  10.1 64位',
            'product_code': '',
            'is_subscribed': False,
            'progress': '100%',
            'creation_time': '2015-06-19T07:25:42Z',
            'usage': 'instance',
            'is_copied': False
        }
        self._validate_extras(expected_extra, image.extra)
        expected_dev_mappings = {
            'snapshot_id': '',
            'size': 20,
            'device': '/dev/xvda',
            'format': '',
            'import_oss_bucket': '',
            'import_oss_object': ''
        }
        self._validate_extras(expected_dev_mappings,
                              image.extra['disk_device_mappings'])

    def test_list_images_with_ex_image_ids(self):
        ECSMockHttp.type = 'list_images_ex_image_ids'
        self.driver.list_images(location=self.fake_location,
                                ex_image_ids=[self.fake_image.id,
                                              'not-existed'])

    def test_list_images_with_ex_image_ids_type_exception(self):
        self.assertRaises(AttributeError, self.driver.list_images,
                          location=self.fake_location,
                          ex_image_ids={'image_ids': 'id1,id2'})

    def test_list_images_with_ex_filters(self):
        ECSMockHttp.type = 'list_images_ex_filters'
        ex_filters = {'Status': 'Available'}
        self.driver.list_images(location=self.fake_location,
                                ex_filters=ex_filters)

    def test_list_images_multiple_pages(self):
        ECSMockHttp.type = 'list_images_pages'
        images = self.driver.list_images()
        self.assertEqual(2, len(images))

    def test_create_image(self):
        self.image_name = 'fake-image1'
        self.description = 'description'
        self.image_version = '1.0.0'
        self.client_token = 'client_token'
        image = self.driver.create_image(None, self.image_name,
                                         self.description,
                                         ex_snapshot_id=self.fake_snapshot.id,
                                         ex_image_version=self.image_version,
                                         ex_client_token=self.client_token)
        self.assertIsNotNone(image)

    def test_creaet_image_exception(self):
        self.assertRaises(AttributeError, self.driver.create_image,
                          None, None)

    def test_delete_image(self):
        result = self.driver.delete_image(self.fake_image)
        self.assertTrue(result)

    def test_get_image(self):
        ECSMockHttp.type = 'get_image'
        image = self.driver.get_image(self.fake_image.id)
        self.assertIsNotNone(image)

    def test_get_image_not_found_exception(self):
        ECSMockHttp.type = 'get_image_not_found'
        self.assertRaises(LibcloudError, self.driver.get_image,
                          self.fake_image.id)

    def test_copy_image(self):
        self.image_name = 'copied-image1'
        self.description = 'description'
        self.dest_region = 'cn-hangzhou'
        self.client_token = 'client-token'
        image = self.driver.copy_image(
            self.region, self.fake_image,
            self.image_name,
            description=self.description,
            ex_destination_region_id=self.dest_region,
            ex_client_token=self.client_token)
        self.assertIsNotNone(image)

    def test_copy_image_in_the_same_region(self):
        ECSMockHttp.type = 'copy_image_same_region'
        image = self.driver.copy_image(self.region, self.fake_image, None)
        self.assertIsNotNone(image)

    def test_ex_list_security_groups(self):
        sgs = self.driver.ex_list_security_groups()
        self.assertEqual(1, len(sgs))
        sg = sgs[0]
        self.assertEqual('sg-28ou0f3xa', sg.id)
        self.assertEqual('sg-28ou0f3xa', sg.name)
        self.assertEqual('System created security group.', sg.description)
        self.assertEqual('', sg.vpc_id)
        self.assertEqual('2015-06-26T08:35:30Z', sg.creation_time)

    def test_ex_list_security_groups_with_ex_filters(self):
        ECSMockHttp.type = 'list_sgs_filters'
        self.vpc_id = 'vpc1'
        ex_filters = {'VpcId': self.vpc_id}
        sgs = self.driver.ex_list_security_groups(ex_filters=ex_filters)
        self.assertEqual(1, len(sgs))

    def test_ex_list_zones(self):
        zones = self.driver.ex_list_zones()
        self.assertEqual(1, len(zones))
        zone = zones[0]
        self.assertEqual('cn-qingdao-b', zone.id)
        self.assertEqual('青岛可用区B', zone.name)
        self.assertEqual(self.driver, zone.driver)
        self.assertIsNotNone(zone.available_resource_types)
        self.assertEqual('IoOptimized', zone.available_resource_types[0])
        self.assertIsNotNone(zone.available_instance_types)
        self.assertEqual('ecs.m2.medium', zone.available_instance_types[0])
        self.assertIsNotNone(zone.available_disk_categories)
        self.assertEqual('cloud_ssd', zone.available_disk_categories[0])


class ECSMockHttp(MockHttpTestCase):
    fixtures = ComputeFileFixtures('ecs')

    def _DescribeInstances(self, method, url, body, headers):
        resp_body = self.fixtures.load('describe_instances.xml')
        return (httplib.OK, resp_body, {}, httplib.responses[httplib.OK])

    def _list_nodes_ex_node_ids_DescribeInstances(self, method, url, body,
                                                  headers):
        params = {'InstanceIds': '["i-28n7dkvov", "not-existed-id"]'}
        self.assertUrlContainsQueryParams(url, params)
        return self._DescribeInstances(method, url, body, headers)

    def _list_nodes_ex_filters_DescribeInstances(self, method, url, body,
                                                 headers):
        params = {'ZoneId': self.test.zone}
        self.assertUrlContainsQueryParams(url, params)
        return self._DescribeInstances(method, url, body, headers)

    def _DescribeInstanceTypes(self, method, url, body, headers):
        resp_body = self.fixtures.load('describe_instance_types.xml')
        return (httplib.OK, resp_body, {}, httplib.responses[httplib.OK])

    def _DescribeRegions(self, method, url, body, headers):
        resp_body = self.fixtures.load('describe_regions.xml')
        return (httplib.OK, resp_body, {}, httplib.responses[httplib.OK])

    def _create_node_CreateInstance(self, method, url, body, headers):
        params = {'SecurityGroupId': 'sg-28ou0f3xa',
                  'Description': 'description',
                  'InternetChargeType': 'PayByTraffic',
                  'InternetMaxBandwidthOut': '1',
                  'InternetMaxBandwidthIn': '200',
                  'HostName': 'hostname',
                  'Password': 'password',
                  'IoOptimized': 'true',
                  'SystemDisk.Category': 'cloud',
                  'SystemDisk.DiskName': 'root',
                  'SystemDisk.Description': 'sys',
                  'VSwitchId': 'vswitch-id1',
                  'PrivateIpAddress': '1.1.1.2',
                  'ClientToken': 'client_token'}
        self.assertUrlContainsQueryParams(url, params)
        resp_body = self.fixtures.load('create_instance.xml')
        return (httplib.OK, resp_body, {}, httplib.responses[httplib.OK])

    def _create_node_DescribeInstances(self, method, url, body, headers):
        resp_body = self.fixtures.load('create_node_describe_instances.xml')
        return (httplib.OK, resp_body, {}, httplib.responses[httplib.OK])

    def _create_node_StartInstance(self, method, url, body, headers):
        resp_body = self.fixtures.load('start_instance.xml')
        return (httplib.OK, resp_body, {}, httplib.responses[httplib.OK])

    def _create_node_with_data_CreateInstance(self, method, url, body,
                                              headers):
        params = {'SecurityGroupId': 'sg-28ou0f3xa',
                  'DataDisk.1.Size': '5',
                  'DataDisk.1.Category': 'cloud',
                  'DataDisk.1.DiskName': 'data1',
                  'DataDisk.1.Description': 'description',
                  'DataDisk.1.Device': '/dev/xvdb',
                  'DataDisk.1.DeleteWithInstance': 'true'}
        self.assertUrlContainsQueryParams(url, params)
        resp_body = self.fixtures.load('create_instance.xml')
        return (httplib.OK, resp_body, {}, httplib.responses[httplib.OK])

    def _create_node_with_data_DescribeInstances(self, method, url, body,
                                                 headers):
        resp_body = self.fixtures.load('create_node_describe_instances.xml')
        return (httplib.OK, resp_body, {}, httplib.responses[httplib.OK])

    def _create_node_with_data_StartInstance(self, method, url, body,
                                             headers):
        resp_body = self.fixtures.load('start_instance.xml')
        return (httplib.OK, resp_body, {}, httplib.responses[httplib.OK])

    def _reboot_node_RebootInstance(self, method, url, body, headers):
        node_id = self.test.fake_node.id
        self.assertUrlContainsQueryParams(url, {'InstanceId': node_id,
                                                'ForceStop': 'false'})
        resp_body = self.fixtures.load('reboot_instance.xml')
        return (httplib.OK, resp_body, {}, httplib.responses[httplib.OK])

    def _reboot_node_DescribeInstances(self, method, url, body, headers):
        resp_body = self.fixtures.load('reboot_node_describe_instances.xml')
        return (httplib.OK, resp_body, {}, httplib.responses[httplib.OK])

    def _reboot_node_force_stop_RebootInstance(self, method, url, body,
                                               headers):
        node_id = self.test.fake_node.id
        self.assertUrlContainsQueryParams(url, {'InstanceId': node_id,
                                                'ForceStop': 'true'})
        resp_body = self.fixtures.load('reboot_instance.xml')
        return (httplib.OK, resp_body, {}, httplib.responses[httplib.OK])

    def _reboot_node_force_stop_DescribeInstances(self, method, url, body,
                                                  headers):
        resp_body = self.fixtures.load('reboot_node_describe_instances.xml')
        return (httplib.OK, resp_body, {}, httplib.responses[httplib.OK])

    def _destroy_node_DescribeInstances(self, method, url, body, headers):
        resp_body = self.fixtures.load('destroy_node_describe_instances.xml')
        return (httplib.OK, resp_body, {}, httplib.responses[httplib.OK])

    def _destroy_node_DeleteInstance(self, method, url, body, headers):
        node_id = self.test.fake_node.id
        self.assertUrlContainsQueryParams(url, {'InstanceId': node_id})
        resp_body = self.fixtures.load('delete_instance.xml')
        return (httplib.OK, resp_body, {}, httplib.responses[httplib.OK])

    def _start_node_StartInstance(self, method, url, body, headers):
        node_id = self.test.fake_node.id
        self.assertUrlContainsQueryParams(url, {'InstanceId': node_id})
        resp_body = self.fixtures.load('start_instance.xml')
        return (httplib.OK, resp_body, {}, httplib.responses[httplib.OK])

    def _start_node_DescribeInstances(self, method, url, body, headers):
        resp_body = self.fixtures.load('reboot_node_describe_instances.xml')
        return (httplib.OK, resp_body, {}, httplib.responses[httplib.OK])

    def _stop_node_StopInstance(self, method, url, body, headers):
        node_id = self.test.fake_node.id
        self.assertUrlContainsQueryParams(url, {'InstanceId': node_id,
                                                'ForceStop': 'false'})
        resp_body = self.fixtures.load('stop_instance.xml')
        return (httplib.OK, resp_body, {}, httplib.responses[httplib.OK])

    def _stop_node_DescribeInstances(self, method, url, body, headers):
        resp_body = self.fixtures.load('stop_node_describe_instances.xml')
        return (httplib.OK, resp_body, {}, httplib.responses[httplib.OK])

    def _stop_node_force_stop_StopInstance(self, method, url, body, headers):
        node_id = self.test.fake_node.id
        self.assertUrlContainsQueryParams(url, {'InstanceId': node_id,
                                                'ForceStop': 'true'})
        resp_body = self.fixtures.load('stop_instance.xml')
        return (httplib.OK, resp_body, {}, httplib.responses[httplib.OK])

    def _stop_node_force_stop_DescribeInstances(self, method, url, body,
                                                headers):
        resp_body = self.fixtures.load('stop_node_describe_instances.xml')
        return (httplib.OK, resp_body, {}, httplib.responses[httplib.OK])

    def _DescribeDisks(self, method, url, body, headers):
        resp_body = self.fixtures.load('describe_disks.xml')
        return (httplib.OK, resp_body, {}, httplib.responses[httplib.OK])

    def _list_volumes_ex_volume_ids_DescribeDisks(self, method, url, body,
                                                  headers):
        region = self.test.region
        params = {'DiskIds': '["i-28n7dkvov", "not-existed-id"]',
                  'RegionId': region}
        self.assertUrlContainsQueryParams(url, params)
        return self._DescribeInstances(method, url, body, headers)

    def _list_volumes_ex_filters_DescribeDisks(self, method, url, body,
                                               headers):
        params = {'InstanceId': self.test.fake_node.id}
        self.assertUrlContainsQueryParams(url, params)
        return self._DescribeDisks(method, url, body, headers)

    def _DescribeSnapshots(self, method, url, body, headers):
        region = self.test.region
        volume_id = self.test.fake_volume.id
        params = {'RegionId': region,
                  'DiskId': volume_id}
        self.assertUrlContainsQueryParams(url, params)
        resp_body = self.fixtures.load('describe_snapshots.xml')
        return (httplib.OK, resp_body, {}, httplib.responses[httplib.OK])

    def _list_volume_snapshots_ex_snapshot_ids_DescribeSnapshots(
            self, method, url, body, headers):
        params = {'RegionId': self.test.region,
                  'SnapshotIds': '["fake-snapshot1"]'}
        self.assertUrlContainsQueryParams(url, params)
        return self._DescribeSnapshots(method, url, body, headers)

    def _list_volume_snapshots_ex_filters_DescribeSnapshots(self, method, url, body,
                                                            headers):
        params = {'InstanceId': self.test.fake_node.id}
        self.assertUrlContainsQueryParams(url, params)
        return self._DescribeSnapshots(method, url, body, headers)

    def _create_volume_CreateDisk(self, method, url, body, headers):
        params = {'RegionId': self.test.region,
                  'DiskName': self.test.volume_name,
                  'Size': str(self.test.volume_size),
                  'ZoneId': self.test.zone,
                  'SnapshotId': self.test.fake_snapshot.id,
                  'Description': self.test.description,
                  'DiskCategory': self.test.disk_category,
                  'ClientToken': self.test.client_token}
        self.assertUrlContainsQueryParams(url, params)
        resp_body = self.fixtures.load('create_disk.xml')
        return (httplib.OK, resp_body, {}, httplib.responses[httplib.OK])

    def _create_volume_DescribeDisks(self, method, url, body, headers):
        resp_body = self.fixtures.load('create_volume_describe_disks.xml')
        return (httplib.OK, resp_body, {}, httplib.responses[httplib.OK])

    def _create_volume_snapshot_CreateSnapshot(self, method, url, body,
                                               headers):
        params = {'DiskId': self.test.fake_volume.id,
                  'SnapshotName': self.test.snapshot_name,
                  'Description': self.test.description,
                  'ClientToken': self.test.client_token}
        self.assertUrlContainsQueryParams(url, params)
        resp_body = self.fixtures.load('create_snapshot.xml')
        return (httplib.OK, resp_body, {}, httplib.responses[httplib.OK])

    def _create_volume_snapshot_DescribeSnapshots(self, method, url, body,
                                                  headers):
        resp_body = self.fixtures.load('describe_snapshots.xml')
        return (httplib.OK, resp_body, {}, httplib.responses[httplib.OK])

    def _AttachDisk(self, method, url, body, headers):
        delete_with_instance = str(self.test.delete_with_instance).lower()
        params = {'InstanceId': self.test.fake_node.id,
                  'DiskId': self.test.fake_volume.id,
                  'Device': self.test.device,
                  'DeleteWithInstance': delete_with_instance}
        self.assertUrlContainsQueryParams(url, params)
        resp_body = self.fixtures.load('attach_disk.xml')
        return (httplib.OK, resp_body, {}, httplib.responses[httplib.OK])

    def _DetachDisk(self, method, url, body, headers):
        params = {'DiskId': self.test.fake_volume.id,
                  'InstanceId': self.test.instance_id}
        self.assertUrlContainsQueryParams(url, params)
        resp_body = self.fixtures.load('detach_disk.xml')
        return (httplib.OK, resp_body, {}, httplib.responses[httplib.OK])

    def _detach_volume_DescribeDisks(self, method, url, body, headers):
        params = {'DiskIds': '["' + self.test.fake_volume.id + '"]'}
        self.assertUrlContainsQueryParams(url, params)
        resp_body = self.fixtures.load('detach_volume_describe_disks.xml')
        return (httplib.OK, resp_body, {}, httplib.responses[httplib.OK])

    def _detach_volume_DetachDisk(self, method, url, body, headers):
        params = {'DiskId': self.test.fake_volume.id,
                  'InstanceId': 'i-28whl2nj2'}
        self.assertUrlContainsQueryParams(url, params)
        resp_body = self.fixtures.load('detach_disk.xml')
        return (httplib.OK, resp_body, {}, httplib.responses[httplib.OK])

    def _destroy_volume_DescribeDisks(self, method, url, body, headers):
        params = {'DiskIds': '["' + self.test.fake_volume.id + '"]'}
        self.assertUrlContainsQueryParams(url, params)
        resp_body = self.fixtures.load('destroy_volume_describe_disks.xml')
        return (httplib.OK, resp_body, {}, httplib.responses[httplib.OK])

    def _destroy_volume_DeleteDisk(self, method, url, body, headers):
        params = {'DiskId': self.test.fake_volume.id}
        self.assertUrlContainsQueryParams(url, params)
        resp_body = self.fixtures.load('delete_disk.xml')
        return (httplib.OK, resp_body, {}, httplib.responses[httplib.OK])

    def _destroy_volume_state_DescribeDisks(self, method, url, body, headers):
        return self._detach_volume_DescribeDisks(method, url, body, headers)

    def _DeleteSnapshot(self, method, url, body, header):
        params = {'SnapshotId': self.test.fake_snapshot.id}
        self.assertUrlContainsQueryParams(url, params)
        resp_body = self.fixtures.load('delete_snapshot.xml')
        return (httplib.OK, resp_body, {}, httplib.responses[httplib.OK])

    def _DescribeImages(self, method, url, body, headers):
        params = {'RegionId': self.test.fake_location.id}
        self.assertUrlContainsQueryParams(url, params)
        resp_body = self.fixtures.load('describe_images.xml')
        return (httplib.OK, resp_body, {}, httplib.responses[httplib.OK])

    def _list_images_pages_DescribeImages(self, method, url, body, headers):
        if 'PageNumber=2' in url:
            resp_body = self.fixtures.load('pages_describe_images_page2.xml')
        else:
            resp_body = self.fixtures.load('pages_describe_images.xml')
        return (httplib.OK, resp_body, {}, httplib.responses[httplib.OK])

    def _list_images_ex_image_ids_DescribeImages(self, method, url, body,
                                                 headers):
        params = {'ImageId': self.test.fake_image.id + ',not-existed'}
        self.assertUrlContainsQueryParams(url, params)
        return self._DescribeImages(method, url, body, headers)

    def _list_images_ex_filters_DescribeImages(self, method, url, body,
                                               headers):
        params = {'Status': 'Available'}
        self.assertUrlContainsQueryParams(url, params)
        return self._DescribeImages(method, url, body, headers)

    def _CreateImage(self, method, url, body, headers):
        params = {'RegionId': self.test.region,
                  'ImageName': self.test.image_name,
                  'Description': self.test.description,
                  'SnapshotId': self.test.fake_snapshot.id,
                  'ImageVersion': self.test.image_version,
                  'ClientToken': self.test.client_token}
        self.assertUrlContainsQueryParams(url, params)
        resp_body = self.fixtures.load('create_image.xml')
        return (httplib.OK, resp_body, {}, httplib.responses[httplib.OK])

    def _DeleteImage(self, method, url, body, headers):
        params = {'RegionId': self.test.region,
                  'ImageId': self.test.fake_image.id}
        self.assertUrlContainsQueryParams(url, params)
        resp_body = self.fixtures.load('delete_image.xml')
        return (httplib.OK, resp_body, {}, httplib.responses[httplib.OK])

    def _get_image_DescribeImages(self, method, url, body, headers):
        params = {'RegionId': self.test.region,
                  'ImageId': self.test.fake_image.id}
        self.assertUrlContainsQueryParams(url, params)
        resp_body = self.fixtures.load('describe_images.xml')
        return (httplib.OK, resp_body, {}, httplib.responses[httplib.OK])

    def _get_image_not_found_DescribeImages(self, method, url, body, headers):
        params = {'RegionId': self.test.region,
                  'ImageId': self.test.fake_image.id}
        self.assertUrlContainsQueryParams(url, params)
        resp_body = self.fixtures.load('get_image_describe_images.xml')
        return (httplib.OK, resp_body, {}, httplib.responses[httplib.OK])

    def _CopyImage(self, method, url, body, headers):
        params = {'RegionId': self.test.region,
                  'ImageId': self.test.fake_image.id,
                  'DestinationRegionId': self.test.dest_region,
                  'DestinationImageName': self.test.image_name,
                  'DestinationDescription': self.test.description,
                  'ClientToken': self.test.client_token}
        self.assertUrlContainsQueryParams(url, params)
        resp_body = self.fixtures.load('copy_image.xml')
        return (httplib.OK, resp_body, {}, httplib.responses[httplib.OK])

    def _copy_image_same_region_CopyImage(self, method, url, body, headers):
        params = {'RegionId': self.test.region,
                  'ImageId': self.test.fake_image.id,
                  'DestinationRegionId': self.test.region}
        self.assertUrlContainsQueryParams(url, params)
        resp_body = self.fixtures.load('copy_image.xml')
        return (httplib.OK, resp_body, {}, httplib.responses[httplib.OK])

    def _copy_image_same_region_DescribeImages(self, method, url, body,
                                               headers):
        return self._DescribeImages(method, url, body, headers)

    def _DescribeSecurityGroups(self, method, url, body, headers):
        params = {'RegionId': self.test.region}
        self.assertUrlContainsQueryParams(url, params)
        resp_body = self.fixtures.load('describe_security_groups.xml')
        return (httplib.OK, resp_body, {}, httplib.responses[httplib.OK])

    def _list_sgs_filters_DescribeSecurityGroups(self, method, url, body,
                                                 headers):
        params = {'VpcId': self.test.vpc_id}
        self.assertUrlContainsQueryParams(url, params)
        return self._DescribeSecurityGroups(method, url, body, headers)

    def _create_sg_CreateSecurityGroup(self, method, url, body, headers):
        params = {'RegionId': self.test.region,
                  'Description': 'description',
                  'ClientToken': 'clientToken'}
        self.assertUrlContainsQueryParams(url, params)
        resp_body = self.fixtures.load('create_security_group.xml')
        return (httplib.OK, resp_body, {}, httplib.responses[httplib.OK])

    def _delete_sg_by_id_DeleteSecurityGroup(self, method, url, body, headers):
        params = {'RegionId': self.test.region,
                  'SecurityGroupId': 'sg-fakeSecurityGroupId'}
        self.assertUrlContainsQueryParams(url, params)
        resp_body = self.fixtures.load('delete_security_group_by_id.xml')
        return (httplib.OK, resp_body, {}, httplib.responses[httplib.OK])

    def _list_sgas_DescribeSecurityGroupAttributes(self, method, url, body, headers):
        params = {'RegionId': self.test.region,
                  'SecurityGroupId': 'sg-fakeSecurityGroupId',
                  'NicType': 'internet'}
        self.assertUrlContainsQueryParams(url, params)
        resp_body = self.fixtures.load('describe_security_group_attributes.xml')
        return (httplib.OK, resp_body, {}, httplib.responses[httplib.OK])

    def _DescribeZones(self, method, url, body, headers):
        resp_body = self.fixtures.load('describe_zones.xml')
        return (httplib.OK, resp_body, {}, httplib.responses[httplib.OK])


if __name__ == '__main__':
    sys.exit(unittest.main())



# Licensed to the Apache Software Foundation (ASF) under one or more
# contributor license agreements.  See the NOTICE file distributed with
# this work for additional information regarding copyright ownership.
# The ASF licenses this file to You under the Apache License, Version 2.0
# (the "License"); you may not use this file except in compliance with
# the License.  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import sys

from libcloud.utils.py3 import httplib
from libcloud.test import MockHttp
from libcloud.test.file_fixtures import ComputeFileFixtures
from libcloud.compute.types import Provider
from libcloud.compute.types import NodeState
from libcloud.compute.providers import get_driver
from libcloud.test import unittest
from libcloud.test.secrets import PROFIT_BRICKS_PARAMS


class ProfitBricksTests(unittest.TestCase):

    def setUp(self):
        ProfitBricks = get_driver(Provider.PROFIT_BRICKS)
        ProfitBricks.connectionCls.conn_classes = (None, ProfitBricksMockHttp)
        self.driver = ProfitBricks(*PROFIT_BRICKS_PARAMS)

    ''' Server Function Tests
    '''
    def test_list_nodes(self):
        nodes = self.driver.list_nodes()

        self.assertEqual(len(nodes), 3)

        node = nodes[0]
        self.assertEqual(node.id, "c8e57d7b-e731-46ad-a913-1828c0562246")
        self.assertEqual(node.name, "server001")
        self.assertEqual(node.state, NodeState.RUNNING)
        self.assertEqual(node.public_ips, ['162.254.25.197'])
        self.assertEqual(node.private_ips, ['10.10.108.12', '10.13.198.11'])
        self.assertEqual(node.extra['datacenter_id'], "e1e8ec0d-b47f-4d39-a91b-6e885483c899")
        self.assertEqual(node.extra['datacenter_version'], "5")
        self.assertEqual(node.extra['provisioning_state'], NodeState.RUNNING)
        self.assertEqual(node.extra['creation_time'], "2014-07-14T20:52:20.839Z")
        self.assertEqual(node.extra['last_modification_time'], "2014-07-14T22:11:09.324Z")
        self.assertEqual(node.extra['os_type'], "LINUX")
        self.assertEqual(node.extra['availability_zone'], "ZONE_1")

    def test_ex_describe_node(self):
        image = type('NodeImage', (object,),
                     dict(id="cd59b162-0289-11e4-9f63-52540066fee9",
                          name="Debian-7-server-2014-07-01"))
        size = type('NodeSize', (object,),
                    dict(id="2",
                         name="Small Instance",
                         ram=2048,
                         disk=50,
                         extra={'cores': 1}))

        node = self.driver.create_node(name="SPC-Server",
                                       image=image,
                                       size=size)

        self.assertEqual(node.id, "7b18b85f-cc93-4c2d-abcc-5ce732d35750")

    def test_reboot_node(self):
        node = type('Node', (object,),
                    dict(id="c8e57d7b-e731-46ad-a913-1828c0562246"))
        reboot = self.driver.reboot_node(node=node)

        self.assertTrue(reboot)

    def test_ex_stop_node(self):
        node = type('Node', (object,),
                    dict(id="c8e57d7b-e731-46ad-a913-1828c0562246"))
        stop = self.driver.ex_stop_node(node=node)

        self.assertTrue(stop)

    def test_ex_start_node(self):
        node = type('Node', (object,),
                    dict(id="c8e57d7b-e731-46ad-a913-1828c0562246"))
        start = self.driver.ex_start_node(node=node)

        self.assertTrue(start)

    def test_destroy_node(self):
        node = type('Node', (object,),
                    dict(id="c8e57d7b-e731-46ad-a913-1828c0562246"))
        destroy = self.driver.destroy_node(node=node)

        self.assertTrue(destroy)

    def test_ex_update_node(self):
        node = type('Node', (object,),
                    dict(id="c8e57d7b-e731-46ad-a913-1828c0562246"))

        zone = type('ExProfitBricksAvailabilityZone', (object,),
                    dict(name="ZONE_2"))

        update = self.driver.ex_update_node(node=node, ram=2048, cores=2, name="server002", availability_zone=zone)

        self.assertTrue(update)

    ''' Volume Function Tests
    '''
    def test_list_volumes(self):
        volumes = self.driver.list_volumes()

        self.assertEqual(len(volumes), 4)

        volume = volumes[0]
        self.assertEqual(volume.id, "453582cf-8d54-4ec8-bc0b-f9962f7fd232")
        self.assertEqual(volume.name, "storage001")
        self.assertEqual(volume.size, 50)
        self.assertEqual(volume.extra['server_id'], "ebee7d83-912b-42f1-9b62-b953351a7e29")
        self.assertEqual(volume.extra['provisioning_state'], NodeState.RUNNING)
        self.assertEqual(volume.extra['creation_time'], "2014-07-15T03:19:38.252Z")
        self.assertEqual(volume.extra['last_modification_time'], "2014-07-15T03:28:58.724Z")
        self.assertEqual(volume.extra['image_id'], "d2f627c4-0289-11e4-9f63-52540066fee9")
        self.assertEqual(volume.extra['image_name'], "CentOS-6-server-2014-07-01")
        self.assertEqual(volume.extra['datacenter_id'], "06eac419-c2b3-4761-aeb9-10efdd2cf292")

    def test_create_volume(self):
        datacenter = type('Datacenter', (object,),
                          dict(id="8669a69f-2274-4520-b51e-dbdf3986a476"))

        image = type('NodeImage', (object,),
                     dict(id="cd59b162-0289-11e4-9f63-52540066fee9"))

        create = self.driver.create_volume(name="StackPointCloudStorage001",
                                           size=50,
                                           ex_datacenter=datacenter,
                                           ex_image=image)

        self.assertTrue(create)

    def test_attach_volume_general(self):
        volume = type('StorageVolume', (object,),
                      dict(id="8669a69f-2274-4520-b51e-dbdf3986a476"))

        node = type('Node', (object,),
                    dict(id="cd59b162-0289-11e4-9f63-52540066fee9"))

        attach = self.driver.attach_volume(node=node,
                                           volume=volume,
                                           device=None, ex_bus_type=None)

        self.assertTrue(attach)

    def test_attach_volume_device_defined(self):
        volume = type('StorageVolume', (object,),
                      dict(id="8669a69f-2274-4520-b51e-dbdf3986a476"))

        node = type('Node', (object,),
                    dict(id="cd59b162-0289-11e4-9f63-52540066fee9"))

        attach = self.driver.attach_volume(node=node, volume=volume, device=1, ex_bus_type=None)

        self.assertTrue(attach)

    def test_attach_volume_bus_type_defined(self):
        volume = type('StorageVolume', (object,),
                      dict(id="8669a69f-2274-4520-b51e-dbdf3986a476"))

        node = type('Node', (object,),
                    dict(id="cd59b162-0289-11e4-9f63-52540066fee9"))

        attach = self.driver.attach_volume(node=node,
                                           volume=volume,
                                           device=None,
                                           ex_bus_type="IDE")

        self.assertTrue(attach)

    def test_attach_volume_options_defined(self):
        volume = type('StorageVolume', (object,),
                      dict(id="8669a69f-2274-4520-b51e-dbdf3986a476"))

        node = type('Node', (object,),
                    dict(id="cd59b162-0289-11e4-9f63-52540066fee9"))

        attach = self.driver.attach_volume(node=node, volume=volume,
                                           device=1, ex_bus_type="IDE")

        self.assertTrue(attach)

    def test_detach_volume(self):
        volume = type('StorageVolume', (object,),
                      dict(id="8669a69f-2274-4520-b51e-dbdf3986a476",
                           extra={'server_id': "cd59b162-0289-11e4-9f63-52540066fee9"}
                           ))

        attach = self.driver.detach_volume(volume=volume)

        self.assertTrue(attach)

    def test_destroy_volume(self):
        volume = type('StorageVolume', (object,),
                      dict(id="8669a69f-2274-4520-b51e-dbdf3986a476"))

        destroy = self.driver.destroy_volume(volume=volume)

        self.assertTrue(destroy)

    def test_update_volume(self):
        volume = type('StorageVolume', (object,),
                      dict(id="8669a69f-2274-4520-b51e-dbdf3986a476"))

        destroy = self.driver.ex_update_volume(volume=volume)

        self.assertTrue(destroy)

    def test_ex_describe_volume(self):
        describe = self.driver.ex_describe_volume(volume_id="8669a69f-2274-4520-b51e-dbdf3986a476")

        self.assertEqual(describe.id, "00d0b9e7-e016-456f-85a0-517aa9a34bf5")
        self.assertEqual(describe.size, 50)
        self.assertEqual(describe.name, "StackPointCloud-Volume")
        self.assertEqual(describe.extra['provisioning_state'], NodeState.RUNNING)

    ''' Image Function Tests
    '''
    def test_list_images(self):
        images = self.driver.list_images()

        self.assertEqual(len(images), 3)

        image = images[0]
        self.assertEqual(image.extra['cpu_hotpluggable'], "false")
        self.assertEqual(image.id, "03b6c3e7-f2ad-11e3-a036-52540066fee9")
        self.assertEqual(image.name, "windows-2012-r2-server-2014-06")
        self.assertEqual(image.extra['image_size'], "11264")
        self.assertEqual(image.extra['image_type'], "HDD")
        self.assertEqual(image.extra['memory_hotpluggable'], "false")
        self.assertEqual(image.extra['os_type'], "WINDOWS")
        self.assertEqual(image.extra['public'], "true")
        self.assertEqual(image.extra['location'], None)
        self.assertEqual(image.extra['writeable'], "true")

    ''' Datacenter Function Tests
    '''
    def test_ex_create_datacenter(self):
        datacenter = self.driver.ex_create_datacenter(name="StackPointCloud",
                                                      location="us/la")

        self.assertEqual(datacenter.id, '0c793dd1-d4cd-4141-86f3-8b1a24b2d604')
        self.assertEqual(datacenter.extra['location'], 'us/las')
        self.assertEqual(datacenter.version, '1')

    def test_ex_destroy_datacenter(self):
        datacenter = type('Datacenter', (object,),
                          dict(id="8669a69f-2274-4520-b51e-dbdf3986a476"))
        destroy = self.driver.ex_destroy_datacenter(datacenter=datacenter)

        self.assertTrue(destroy)

    def test_ex_describe_datacenter(self):
        datacenter = type('Datacenter', (object,),
                          dict(id="d96dfafc-9a8c-4c0e-8a0c-857a15db572d"))
        describe = self.driver.ex_describe_datacenter(datacenter_id=datacenter.id)

        self.assertEqual(describe.id, 'a3e6f83a-8982-4d6a-aebc-60baf5755ede')
        self.assertEqual(describe.name, 'StackPointCloud')
        self.assertEqual(describe.version, '1')
        self.assertEqual(describe.extra['location'], 'us/las')
        self.assertEqual(describe.extra['provisioning_state'], NodeState.RUNNING)

    def test_ex_clear_datacenter(self):
        datacenter = type('Datacenter', (object,),
                          dict(id="8669a69f-2274-4520-b51e-dbdf3986a476"))
        clear = self.driver.ex_clear_datacenter(datacenter=datacenter)

        self.assertTrue(clear)

    def test_ex_list_datacenters(self):
        datacenters = self.driver.ex_list_datacenters()

        self.assertEqual(len(datacenters), 2)

        dc1 = datacenters[0]
        self.assertEqual(dc1.id, "a3e6f83a-8982-4d6a-aebc-60baf5755ede")
        self.assertEqual(dc1.name, "StackPointCloud")
        self.assertEqual(dc1.version, "1")

    def test_ex_rename_datacenter(self):
        datacenter = type('Datacenter', (object,),
                          dict(id="d96dfafc-9a8c-4c0e-8a0c-857a15db572d"))

        update = self.driver.ex_rename_datacenter(datacenter=datacenter,
                                                  name="StackPointCloud")

        self.assertTrue(update)

    def test_list_locations(self):
        locations = self.driver.list_locations()
        self.assertEqual(len(locations), 3)

        locationNamesResult = sorted(list(a.name for a in locations))
        locationNamesExpected = ['de/fkb', 'de/fra', 'us/las']

        self.assertEqual(locationNamesResult, locationNamesExpected)

    ''' Availability Zone Tests
    '''

    def test_ex_list_availability_zones(self):
        zones = self.driver.ex_list_availability_zones()
        self.assertEqual(len(zones), 3)

        zoneNamesResult = sorted(list(a.name for a in zones))
        zoneNamesExpected = ['AUTO', 'ZONE_1', 'ZONE_2']

        self.assertEqual(zoneNamesResult, zoneNamesExpected)

    ''' Interface Tests
    '''

    def test_ex_list_interfaces(self):
        interfaces = self.driver.ex_list_network_interfaces()

        self.assertEqual(len(interfaces), 3)

        interface = interfaces[0]
        self.assertEqual(interface.id, "6b38a4f3-b851-4614-9e3a-5ddff4727727")
        self.assertEqual(interface.name, "StackPointCloud")
        self.assertEqual(interface.state, NodeState.RUNNING)
        self.assertEqual(interface.extra['server_id'], "234f0cf9-1efc-4ade-b829-036456584116")
        self.assertEqual(interface.extra['lan_id'], '3')
        self.assertEqual(interface.extra['internet_access'], 'false')
        self.assertEqual(interface.extra['mac_address'], "02:01:40:47:90:04")
        self.assertEqual(interface.extra['dhcp_active'], "true")
        self.assertEqual(interface.extra['gateway_ip'], None)
        self.assertEqual(interface.extra['ips'], ['10.14.96.11', '162.254.26.14', '162.254.26.15'])

    def test_ex_create_network_interface(self):
        node = type('Node', (object,),
                    dict(id="cd59b162-0289-11e4-9f63-52540066fee9"))

        interface = self.driver.ex_create_network_interface(node=node)
        self.assertEqual(interface.id, '6b38a4f3-b851-4614-9e3a-5ddff4727727')

    def test_ex_destroy_network_interface(self):
        network_interface = type('ProfitBricksNetworkInterface', (object,),
                                 dict(
                                 id="cd59b162-0289-11e4-9f63-52540066fee9"))

        destroy = self.driver.ex_destroy_network_interface(
            network_interface=network_interface)

        self.assertTrue(destroy)

    def test_ex_update_network_interface(self):
        network_interface = type('ProfitBricksNetworkInterface', (object,),
                                 dict(
                                 id="cd59b162-0289-11e4-9f63-52540066fee9"))

        create = self.driver.ex_update_network_interface(
            network_interface=network_interface)

        self.assertTrue(create)

    def test_ex_describe_network_interface(self):
        network_interface = type('ProfitBricksNetworkInterface', (object,),
                                 dict(
                                 id="cd59b162-0289-11e4-9f63-52540066fee9"))

        describe = self.driver.ex_describe_network_interface(network_interface=network_interface)

        self.assertEqual(describe.id, "f1c7a244-2fa6-44ee-8fb6-871f337683a3")
        self.assertEqual(describe.name, None)
        self.assertEqual(describe.state, NodeState.RUNNING)
        self.assertEqual(describe.extra['datacenter_id'], "a3a2e730-0dc3-47e6-bac6-4c056d5e2aee")
        self.assertEqual(describe.extra['datacenter_version'], "6")
        self.assertEqual(describe.extra['server_id'], "c09f4f31-336c-4ad2-9ec7-591778513408")
        self.assertEqual(describe.extra['lan_id'], "1")
        self.assertEqual(describe.extra['internet_access'], "false")
        self.assertEqual(describe.extra['mac_address'], "02:01:96:d7:60:e0")
        self.assertEqual(describe.extra['dhcp_active'], "true")
        self.assertEqual(describe.extra['gateway_ip'], None)
        self.assertEqual(describe.extra['ips'], ['10.10.38.12'])

    def test_list_sizes(self):
        sizes = self.driver.list_sizes()

        self.assertEqual(len(sizes), 7)


class ProfitBricksMockHttp(MockHttp):

    fixtures = ComputeFileFixtures('profitbricks')

    def _1_3_clearDataCenter(self, method, url, body, headers):
        body = self.fixtures.load('ex_clear_datacenter.xml')
        return (httplib.OK, body, {}, httplib.responses[httplib.OK])

    def _1_3_createDataCenter(self, method, url, body, headers):
        body = self.fixtures.load('ex_create_datacenter.xml')
        return (httplib.OK, body, {}, httplib.responses[httplib.OK])

    def _1_3_deleteDataCenter(self, method, url, body, headers):
        body = self.fixtures.load('ex_destroy_datacenter.xml')
        return (httplib.OK, body, {}, httplib.responses[httplib.OK])

    def _1_3_getDataCenter(self, method, url, body, headers):
        body = self.fixtures.load('ex_describe_datacenter.xml')
        return (httplib.OK, body, {}, httplib.responses[httplib.OK])

    def _1_3_getAllDataCenters(self, method, url, body, headers):
        body = self.fixtures.load('ex_list_datacenters.xml')
        return (httplib.OK, body, {}, httplib.responses[httplib.OK])

    def _1_3_updateDataCenter(self, method, url, body, headers):
        body = self.fixtures.load('ex_update_datacenter.xml')
        return (httplib.OK, body, {}, httplib.responses[httplib.OK])

    def _1_3_getAllImages(self, method, url, body, headers):
        body = self.fixtures.load('list_images.xml')
        return (httplib.OK, body, {}, httplib.responses[httplib.OK])

    def _1_3_getAllServers(self, method, url, body, headers):
        body = self.fixtures.load('list_nodes.xml')
        return (httplib.OK, body, {}, httplib.responses[httplib.OK])

    def _1_3_resetServer(self, method, url, body, headers):
        body = self.fixtures.load('reboot_node.xml')
        return (httplib.OK, body, {}, httplib.responses[httplib.OK])

    def _1_3_stopServer(self, method, url, body, headers):
        body = self.fixtures.load('ex_stop_node.xml')
        return (httplib.OK, body, {}, httplib.responses[httplib.OK])

    def _1_3_startServer(self, method, url, body, headers):
        body = self.fixtures.load('ex_start_node.xml')
        return (httplib.OK, body, {}, httplib.responses[httplib.OK])

    def _1_3_deleteServer(self, method, url, body, headers):
        body = self.fixtures.load('destroy_node.xml')
        return (httplib.OK, body, {}, httplib.responses[httplib.OK])

    def _1_3_getAllStorages(self, method, url, body, headers):
        body = self.fixtures.load('list_volumes.xml')
        return (httplib.OK, body, {}, httplib.responses[httplib.OK])

    def _1_3_createStorage(self, method, url, body, headers):
        body = self.fixtures.load('create_volume.xml')
        return (httplib.OK, body, {}, httplib.responses[httplib.OK])

    def _1_3_connectStorageToServer(self, method, url, body, headers):
        body = self.fixtures.load('attach_volume.xml')
        return (httplib.OK, body, {}, httplib.responses[httplib.OK])

    def _1_3_disconnectStorageFromServer(self, method, url, body, headers):
        body = self.fixtures.load('detach_volume.xml')
        return (httplib.OK, body, {}, httplib.responses[httplib.OK])

    def _1_3_deleteStorage(self, method, url, body, headers):
        body = self.fixtures.load('destroy_volume.xml')
        return (httplib.OK, body, {}, httplib.responses[httplib.OK])

    def _1_3_updateStorage(self, method, url, body, headers):
        body = self.fixtures.load('ex_update_volume.xml')
        return (httplib.OK, body, {}, httplib.responses[httplib.OK])

    def _1_3_updateServer(self, method, url, body, headers):
        body = self.fixtures.load('ex_update_node.xml')
        return (httplib.OK, body, {}, httplib.responses[httplib.OK])

    def _1_3_getNic(self, method, url, body, headers):
        body = self.fixtures.load('ex_describe_network_interface.xml')
        return (httplib.OK, body, {}, httplib.responses[httplib.OK])

    def _1_3_getAllNic(self, method, url, body, headers):
        body = self.fixtures.load('ex_list_network_interfaces.xml')
        return (httplib.OK, body, {}, httplib.responses[httplib.OK])

    def _1_3_createNic(self, method, url, body, headers):
        body = self.fixtures.load('ex_list_network_interfaces.xml')
        return (httplib.OK, body, {}, httplib.responses[httplib.OK])

    def _1_3_deleteNic(self, method, url, body, headers):
        body = self.fixtures.load('ex_destroy_network_interface.xml')
        return (httplib.OK, body, {}, httplib.responses[httplib.OK])

    def _1_3_updateNic(self, method, url, body, headers):
        body = self.fixtures.load('ex_update_network_interface.xml')
        return (httplib.OK, body, {}, httplib.responses[httplib.OK])

    def _1_3_getServer(self, method, url, body, headers):
        body = self.fixtures.load('ex_describe_node.xml')
        return (httplib.OK, body, {}, httplib.responses[httplib.OK])

    def _1_3_getStorage(self, method, url, body, headers):
        body = self.fixtures.load('ex_describe_volume.xml')
        return (httplib.OK, body, {}, httplib.responses[httplib.OK])

    def _1_3_createServer(self, method, url, body, headers):
        body = self.fixtures.load('create_node.xml')
        return (httplib.OK, body, {}, httplib.responses[httplib.OK])

if __name__ == '__main__':
    sys.exit(unittest.main())



# Licensed to the Apache Software Foundation (ASF) under one or more
# contributor license agreements.  See the NOTICE file distributed with
# this work for additional information regarding copyright ownership.
# The ASF licenses this file to You under the Apache License, Version 2.0
# (the "License"); you may not use this file except in compliance with
# the License.  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
# Maintainer: Jed Smith <jed@linode.com>
# Based upon code written by Alex Polvi <polvi@cloudkick.com>
#

import sys
import unittest
from libcloud.utils.py3 import httplib

from libcloud.compute.drivers.linode import LinodeNodeDriver
from libcloud.compute.base import Node, NodeAuthPassword
from libcloud.compute.base import NodeAuthSSHKey, StorageVolume

from libcloud.test import MockHttp
from libcloud.test.compute import TestCaseMixin
from libcloud.test.file_fixtures import ComputeFileFixtures


class LinodeTest(unittest.TestCase, TestCaseMixin):
    # The Linode test suite

    def setUp(self):
        LinodeNodeDriver.connectionCls.conn_classes = (None, LinodeMockHttp)
        LinodeMockHttp.use_param = 'api_action'
        self.driver = LinodeNodeDriver('foo')

    def test_list_nodes(self):
        nodes = self.driver.list_nodes()
        self.assertEqual(len(nodes), 1)
        node = nodes[0]
        self.assertEqual(node.id, "8098")
        self.assertEqual(node.name, 'api-node3')
        self.assertEqual(node.extra['PLANID'], '2')
        self.assertTrue('75.127.96.245' in node.public_ips)
        self.assertEqual(node.private_ips, [])

    def test_reboot_node(self):
        # An exception would indicate failure
        node = self.driver.list_nodes()[0]
        self.driver.reboot_node(node)

    def test_destroy_node(self):
        # An exception would indicate failure
        node = self.driver.list_nodes()[0]
        self.driver.destroy_node(node)

    def test_create_node_password_auth(self):
        # Will exception on failure
        self.driver.create_node(name="Test",
                                location=self.driver.list_locations()[0],
                                size=self.driver.list_sizes()[0],
                                image=self.driver.list_images()[6],
                                auth=NodeAuthPassword("test123"))

    def test_create_node_ssh_key_auth(self):
        # Will exception on failure
        node = self.driver.create_node(name="Test",
                                       location=self.driver.list_locations()[
                                           0],
                                       size=self.driver.list_sizes()[0],
                                       image=self.driver.list_images()[6],
                                       auth=NodeAuthSSHKey('foo'))
        self.assertTrue(isinstance(node, Node))

    def test_list_sizes(self):
        sizes = self.driver.list_sizes()
        self.assertEqual(len(sizes), 9)
        for size in sizes:
            self.assertEqual(size.ram, int(size.name.split(" ")[1]))

    def test_list_images(self):
        images = self.driver.list_images()
        self.assertEqual(len(images), 30)

    def test_create_node_response(self):
        # should return a node object
        node = self.driver.create_node(name="node-name",
                                       location=self.driver.list_locations()[
                                           0],
                                       size=self.driver.list_sizes()[0],
                                       image=self.driver.list_images()[0],
                                       auth=NodeAuthPassword("foobar"))
        self.assertTrue(isinstance(node, Node))

    def test_destroy_volume(self):
        # Will exception on failure
        node = self.driver.list_nodes()[0]
        volume = StorageVolume(id=55648, name="test", size=1024,
                               driver=self.driver, extra={"LINODEID": node.id})
        self.driver.destroy_volume(volume)

    def test_ex_create_volume(self):
        # should return a StorageVolume object
        node = self.driver.list_nodes()[0]
        volume = self.driver.ex_create_volume(size=4096,
                                              name="Another test image",
                                              node=node,
                                              fs_type="ext4")
        self.assertTrue(isinstance(volume, StorageVolume))

    def test_ex_list_volumes(self):
        # should return list of StorageVolume objects
        node = self.driver.list_nodes()[0]
        volumes = self.driver.ex_list_volumes(node=node)

        self.assertTrue(isinstance(volumes, list))
        self.assertTrue(isinstance(volumes[0], StorageVolume))
        self.assertEqual(len(volumes), 2)


class LinodeMockHttp(MockHttp):
    fixtures = ComputeFileFixtures('linode')

    def _avail_datacenters(self, method, url, body, headers):
        body = self.fixtures.load('_avail_datacenters.json')
        return (httplib.OK, body, {}, httplib.responses[httplib.OK])

    def _avail_linodeplans(self, method, url, body, headers):
        body = self.fixtures.load('_avail_linodeplans.json')
        return (httplib.OK, body, {}, httplib.responses[httplib.OK])

    def _avail_distributions(self, method, url, body, headers):
        body = self.fixtures.load('_avail_distributions.json')
        return (httplib.OK, body, {}, httplib.responses[httplib.OK])

    def _linode_create(self, method, url, body, headers):
        body = '{"ERRORARRAY":[],"ACTION":"linode.create","DATA":{"LinodeID":8098}}'
        return (httplib.OK, body, {}, httplib.responses[httplib.OK])

    def _linode_disk_create(self, method, url, body, headers):
        body = '{"ERRORARRAY":[],"ACTION":"linode.disk.create","DATA":{"JobID":1298,"DiskID":55647}}'
        return (httplib.OK, body, {}, httplib.responses[httplib.OK])

    def _linode_disk_delete(self, method, url, body, headers):
        body = '{"ERRORARRAY":[],"ACTION":"linode.disk.delete","DATA":{"JobID":1298,"DiskID":55648}}'
        return (httplib.OK, body, {}, httplib.responses[httplib.OK])

    def _linode_disk_createfromdistribution(self, method, url, body, headers):
        body = '{"ERRORARRAY":[],"ACTION":"linode.disk.createFromDistribution","DATA":{"JobID":1298,"DiskID":55647}}'
        return (httplib.OK, body, {}, httplib.responses[httplib.OK])

    def _linode_disk_list(self, method, url, body, headers):
        body = self.fixtures.load('_linode_disk_list.json')
        return (httplib.OK, body, {}, httplib.responses[httplib.OK])

    def _linode_delete(self, method, url, body, headers):
        body = '{"ERRORARRAY":[],"ACTION":"linode.delete","DATA":{"LinodeID":8098}}'
        return (httplib.OK, body, {}, httplib.responses[httplib.OK])

    def _linode_update(self, method, url, body, headers):
        body = '{"ERRORARRAY":[],"ACTION":"linode.update","DATA":{"LinodeID":8098}}'
        return (httplib.OK, body, {}, httplib.responses[httplib.OK])

    def _linode_reboot(self, method, url, body, headers):
        body = '{"ERRORARRAY":[],"ACTION":"linode.reboot","DATA":{"JobID":1305}}'
        return (httplib.OK, body, {}, httplib.responses[httplib.OK])

    def _avail_kernels(self, method, url, body, headers):
        body = self.fixtures.load('_avail_kernels.json')
        return (httplib.OK, body, {}, httplib.responses[httplib.OK])

    def _linode_boot(self, method, url, body, headers):
        body = '{"ERRORARRAY":[],"ACTION":"linode.boot","DATA":{"JobID":1300}}'
        return (httplib.OK, body, {}, httplib.responses[httplib.OK])

    def _linode_config_create(self, method, url, body, headers):
        body = '{"ERRORARRAY":[],"ACTION":"linode.config.create","DATA":{"ConfigID":31239}}'
        return (httplib.OK, body, {}, httplib.responses[httplib.OK])

    def _linode_list(self, method, url, body, headers):
        body = self.fixtures.load('_linode_list.json')
        return (httplib.OK, body, {}, httplib.responses[httplib.OK])

    def _linode_ip_list(self, method, url, body, headers):
        body = self.fixtures.load('_linode_ip_list.json')
        return (httplib.OK, body, {}, httplib.responses[httplib.OK])

    def _batch(self, method, url, body, headers):
        body = self.fixtures.load('_batch.json')
        return (httplib.OK, body, {}, httplib.responses[httplib.OK])


if __name__ == '__main__':
    sys.exit(unittest.main())



# Licensed to the Apache Software Foundation (ASF) under one or more
# contributor license agreements.  See the NOTICE file distributed with
# this work for additional information regarding copyright ownership.
# The ASF licenses this file to You under the Apache License, Version 2.0
# (the "License"); you may not use this file except in compliance with
# the License.  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import sys
import unittest
import zlib
import gzip

from mock import Mock

from libcloud.utils.py3 import httplib, b, StringIO, PY3
from libcloud.common.base import Response, XmlResponse, JsonResponse
from libcloud.common.types import MalformedResponseError


class ResponseClassesTests(unittest.TestCase):
    def setUp(self):
        self._mock_response = Mock()
        self._mock_response.getheaders.return_value = []
        self._mock_response.status = httplib.OK
        self._mock_response._original_data = None
        self._mock_connection = Mock()

    def test_XmlResponse_class(self):
        self._mock_response.read.return_value = '<foo>bar</foo>'
        response = XmlResponse(response=self._mock_response,
                               connection=self._mock_connection)

        parsed = response.parse_body()
        self.assertEqual(parsed.tag, 'foo')
        self.assertEqual(parsed.text, 'bar')

    def test_XmlResponse_class_malformed_response(self):
        self._mock_response.read.return_value = '<foo>'

        try:
            XmlResponse(response=self._mock_response,
                        connection=self._mock_connection)
        except MalformedResponseError:
            pass
        else:
            self.fail('Exception was not thrown')

    def test_XmlResponse_class_zero_length_body_strip(self):
        self._mock_response.read.return_value = ' '

        response = XmlResponse(response=self._mock_response,
                               connection=self._mock_connection)

        parsed = response.parse_body()
        self.assertEqual(parsed, '')

    def test_JsonResponse_class_success(self):
        self._mock_response.read.return_value = '{"foo": "bar"}'
        response = JsonResponse(response=self._mock_response,
                                connection=self._mock_connection)

        parsed = response.parse_body()
        self.assertEqual(parsed, {'foo': 'bar'})

    def test_JsonResponse_class_malformed_response(self):
        self._mock_response.read.return_value = '{"foo": "bar'

        try:
            JsonResponse(response=self._mock_response,
                         connection=self._mock_connection)
        except MalformedResponseError:
            pass
        else:
            self.fail('Exception was not thrown')

    def test_JsonResponse_class_zero_length_body_strip(self):
        self._mock_response.read.return_value = ' '

        response = JsonResponse(response=self._mock_response,
                                connection=self._mock_connection)

        parsed = response.parse_body()
        self.assertEqual(parsed, '')

    def test_deflate_encoding(self):
        original_data = 'foo bar ponies, wooo zlib'
        compressed_data = zlib.compress(b(original_data))

        self._mock_response.read.return_value = compressed_data
        self._mock_response.getheaders.return_value = \
            {'Content-Encoding': 'deflate'}

        response = Response(response=self._mock_response,
                            connection=self._mock_connection)

        body = response.parse_body()
        self.assertEqual(body, original_data)

        self._mock_response.getheaders.return_value = \
            {'Content-Encoding': 'zlib'}

        response = Response(response=self._mock_response,
                            connection=self._mock_connection)

        body = response.parse_body()
        self.assertEqual(body, original_data)

    def test_gzip_encoding(self):
        original_data = 'foo bar ponies, wooo gzip'

        if PY3:
            from io import BytesIO
            string_io = BytesIO()
        else:
            string_io = StringIO()

        stream = gzip.GzipFile(fileobj=string_io, mode='w')
        stream.write(b(original_data))
        stream.close()
        compressed_data = string_io.getvalue()

        self._mock_response.read.return_value = compressed_data
        self._mock_response.getheaders.return_value = \
            {'Content-Encoding': 'gzip'}

        response = Response(response=self._mock_response,
                            connection=self._mock_connection)

        body = response.parse_body()
        self.assertEqual(body, original_data)

        self._mock_response.getheaders.return_value = \
            {'Content-Encoding': 'x-gzip'}

        response = Response(response=self._mock_response,
                            connection=self._mock_connection)

        body = response.parse_body()
        self.assertEqual(body, original_data)


if __name__ == '__main__':
    sys.exit(unittest.main())



# Licensed to the Apache Software Foundation (ASF) under one or more
# contributor license agreements.  See the NOTICE file distributed with
# this work for additional information regarding copyright ownership.
# The ASF licenses this file to You under the Apache License, Version 2.0
# (the "License"); you may not use this file except in compliance with
# the License.  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Base driver for the providers based on the ElasticStack platform -
http://www.elasticstack.com.
"""

import re
import time
import base64

from libcloud.utils.py3 import httplib
from libcloud.utils.py3 import b

try:
    import simplejson as json
except ImportError:
    import json

from libcloud.common.base import ConnectionUserAndKey, JsonResponse
from libcloud.common.types import InvalidCredsError
from libcloud.compute.types import NodeState
from libcloud.compute.base import NodeDriver, NodeSize, Node
from libcloud.compute.base import NodeImage
from libcloud.compute.deployment import ScriptDeployment, SSHKeyDeployment
from libcloud.compute.deployment import MultiStepDeployment


NODE_STATE_MAP = {
    'active': NodeState.RUNNING,
    'dead': NodeState.TERMINATED,
    'dumped': NodeState.TERMINATED,
}

# Default timeout (in seconds) for the drive imaging process
IMAGING_TIMEOUT = 10 * 60

# ElasticStack doesn't specify special instance types, so I just specified
# some plans based on the other provider offerings.
#
# Basically for CPU any value between 500Mhz and 20000Mhz should work,
# 256MB to 8192MB for ram and 1GB to 2TB for disk.
INSTANCE_TYPES = {
    'small': {
        'id': 'small',
        'name': 'Small instance',
        'cpu': 2000,
        'memory': 1700,
        'disk': 160,
        'bandwidth': None,
    },
    'medium': {
        'id': 'medium',
        'name': 'Medium instance',
        'cpu': 3000,
        'memory': 4096,
        'disk': 500,
        'bandwidth': None,
    },
    'large': {
        'id': 'large',
        'name': 'Large instance',
        'cpu': 4000,
        'memory': 7680,
        'disk': 850,
        'bandwidth': None,
    },
    'extra-large': {
        'id': 'extra-large',
        'name': 'Extra Large instance',
        'cpu': 8000,
        'memory': 8192,
        'disk': 1690,
        'bandwidth': None,
    },
    'high-cpu-medium': {
        'id': 'high-cpu-medium',
        'name': 'High-CPU Medium instance',
        'cpu': 5000,
        'memory': 1700,
        'disk': 350,
        'bandwidth': None,
    },
    'high-cpu-extra-large': {
        'id': 'high-cpu-extra-large',
        'name': 'High-CPU Extra Large instance',
        'cpu': 20000,
        'memory': 7168,
        'disk': 1690,
        'bandwidth': None,
    },
}


class ElasticStackException(Exception):
    def __str__(self):
        return self.args[0]

    def __repr__(self):
        return "<ElasticStackException '%s'>" % (self.args[0])


class ElasticStackResponse(JsonResponse):
    def success(self):
        if self.status == 401:
            raise InvalidCredsError()

        return self.status >= 200 and self.status <= 299

    def parse_error(self):
        error_header = self.headers.get('x-elastic-error', '')
        return 'X-Elastic-Error: %s (%s)' % (error_header, self.body.strip())


class ElasticStackNodeSize(NodeSize):
    def __init__(self, id, name, cpu, ram, disk, bandwidth, price, driver):
        self.id = id
        self.name = name
        self.cpu = cpu
        self.ram = ram
        self.disk = disk
        self.bandwidth = bandwidth
        self.price = price
        self.driver = driver

    def __repr__(self):
        return (('<NodeSize: id=%s, name=%s, cpu=%s, ram=%s '
                 'disk=%s bandwidth=%s price=%s driver=%s ...>')
                % (self.id, self.name, self.cpu, self.ram,
                   self.disk, self.bandwidth, self.price, self.driver.name))


class ElasticStackBaseConnection(ConnectionUserAndKey):
    """
    Base connection class for the ElasticStack driver
    """

    host = None
    responseCls = ElasticStackResponse

    def add_default_headers(self, headers):
        headers['Accept'] = 'application/json'
        headers['Content-Type'] = 'application/json'
        headers['Authorization'] = \
            ('Basic %s' % (base64.b64encode(b('%s:%s' % (self.user_id,
                                                         self.key))))
                .decode('utf-8'))
        return headers


class ElasticStackBaseNodeDriver(NodeDriver):
    website = 'http://www.elasticstack.com'
    connectionCls = ElasticStackBaseConnection
    features = {"create_node": ["generates_password"]}

    def reboot_node(self, node):
        # Reboots the node
        response = self.connection.request(
            action='/servers/%s/reset' % (node.id),
            method='POST'
        )
        return response.status == 204

    def destroy_node(self, node):
        # Kills the server immediately
        response = self.connection.request(
            action='/servers/%s/destroy' % (node.id),
            method='POST'
        )
        return response.status == 204

    def list_images(self, location=None):
        # Returns a list of available pre-installed system drive images
        images = []
        for key, value in self._standard_drives.items():
            image = NodeImage(
                id=value['uuid'],
                name=value['description'],
                driver=self.connection.driver,
                extra={
                    'size_gunzipped': value['size_gunzipped']
                }
            )
            images.append(image)

        return images

    def list_sizes(self, location=None):
        sizes = []
        for key, value in INSTANCE_TYPES.items():
            size = ElasticStackNodeSize(
                id=value['id'],
                name=value['name'], cpu=value['cpu'], ram=value['memory'],
                disk=value['disk'], bandwidth=value['bandwidth'],
                price=self._get_size_price(size_id=value['id']),
                driver=self.connection.driver
            )
            sizes.append(size)

        return sizes

    def list_nodes(self):
        # Returns a list of active (running) nodes
        response = self.connection.request(action='/servers/info').object

        nodes = []
        for data in response:
            node = self._to_node(data)
            nodes.append(node)

        return nodes

    def create_node(self, **kwargs):
        """Creates an ElasticStack instance

        @inherits: :class:`NodeDriver.create_node`

        :keyword    name: String with a name for this new node (required)
        :type       name: ``str``

        :keyword    smp: Number of virtual processors or None to calculate
                         based on the cpu speed
        :type       smp: ``int``

        :keyword    nic_model: e1000, rtl8139 or virtio
                               (if not specified, e1000 is used)
        :type       nic_model: ``str``

        :keyword    vnc_password: If set, the same password is also used for
                                  SSH access with user toor,
                                  otherwise VNC access is disabled and
                                  no SSH login is possible.
        :type       vnc_password: ``str``
        """
        size = kwargs['size']
        image = kwargs['image']
        smp = kwargs.get('smp', 'auto')
        nic_model = kwargs.get('nic_model', 'e1000')
        vnc_password = ssh_password = kwargs.get('vnc_password', None)

        if nic_model not in ('e1000', 'rtl8139', 'virtio'):
            raise ElasticStackException('Invalid NIC model specified')

        # check that drive size is not smaller than pre installed image size

        # First we create a drive with the specified size
        drive_data = {}
        drive_data.update({'name': kwargs['name'],
                           'size': '%sG' % (kwargs['size'].disk)})

        response = self.connection.request(action='/drives/create',
                                           data=json.dumps(drive_data),
                                           method='POST').object

        if not response:
            raise ElasticStackException('Drive creation failed')

        drive_uuid = response['drive']

        # Then we image the selected pre-installed system drive onto it
        response = self.connection.request(
            action='/drives/%s/image/%s/gunzip' % (drive_uuid, image.id),
            method='POST'
        )

        if response.status not in (200, 204):
            raise ElasticStackException('Drive imaging failed')

        # We wait until the drive is imaged and then boot up the node
        # (in most cases, the imaging process shouldn't take longer
        # than a few minutes)
        response = self.connection.request(
            action='/drives/%s/info' % (drive_uuid)
        ).object

        imaging_start = time.time()
        while 'imaging' in response:
            response = self.connection.request(
                action='/drives/%s/info' % (drive_uuid)
            ).object

            elapsed_time = time.time() - imaging_start
            if ('imaging' in response and elapsed_time >= IMAGING_TIMEOUT):
                raise ElasticStackException('Drive imaging timed out')

            time.sleep(1)

        node_data = {}
        node_data.update({'name': kwargs['name'],
                          'cpu': size.cpu,
                          'mem': size.ram,
                          'ide:0:0': drive_uuid,
                          'boot': 'ide:0:0',
                          'smp': smp})
        node_data.update({'nic:0:model': nic_model, 'nic:0:dhcp': 'auto'})

        if vnc_password:
            node_data.update({'vnc': 'auto', 'vnc:password': vnc_password})

        response = self.connection.request(
            action='/servers/create', data=json.dumps(node_data),
            method='POST'
        ).object

        if isinstance(response, list):
            nodes = [self._to_node(node, ssh_password) for node in response]
        else:
            nodes = self._to_node(response, ssh_password)

        return nodes

    # Extension methods
    def ex_set_node_configuration(self, node, **kwargs):
        """
        Changes the configuration of the running server

        :param      node: Node which should be used
        :type       node: :class:`Node`

        :param      kwargs: keyword arguments
        :type       kwargs: ``dict``

        :rtype: ``bool``
        """
        valid_keys = ('^name$', '^parent$', '^cpu$', '^smp$', '^mem$',
                      '^boot$', '^nic:0:model$', '^nic:0:dhcp',
                      '^nic:1:model$', '^nic:1:vlan$', '^nic:1:mac$',
                      '^vnc:ip$', '^vnc:password$', '^vnc:tls',
                      '^ide:[0-1]:[0-1](:media)?$',
                      '^scsi:0:[0-7](:media)?$', '^block:[0-7](:media)?$')

        invalid_keys = []
        keys = list(kwargs.keys())
        for key in keys:
            matches = False
            for regex in valid_keys:
                if re.match(regex, key):
                    matches = True
                    break
            if not matches:
                invalid_keys.append(key)

        if invalid_keys:
            raise ElasticStackException(
                'Invalid configuration key specified: %s'
                % (',' .join(invalid_keys))
            )

        response = self.connection.request(
            action='/servers/%s/set' % (node.id), data=json.dumps(kwargs),
            method='POST'
        )

        return (response.status == httplib.OK and response.body != '')

    def deploy_node(self, **kwargs):
        """
        Create a new node, and start deployment.

        @inherits: :class:`NodeDriver.deploy_node`

        :keyword    enable_root: If true, root password will be set to
                                 vnc_password (this will enable SSH access)
                                 and default 'toor' account will be deleted.
        :type       enable_root: ``bool``
        """
        image = kwargs['image']
        vnc_password = kwargs.get('vnc_password', None)
        enable_root = kwargs.get('enable_root', False)

        if not vnc_password:
            raise ValueError('You need to provide vnc_password argument '
                             'if you want to use deployment')

        if (image in self._standard_drives and
                not self._standard_drives[image]['supports_deployment']):
            raise ValueError('Image %s does not support deployment'
                             % (image.id))

        if enable_root:
            script = ("unset HISTFILE;"
                      "echo root:%s | chpasswd;"
                      "sed -i '/^toor.*$/d' /etc/passwd /etc/shadow;"
                      "history -c") % vnc_password
            root_enable_script = ScriptDeployment(script=script,
                                                  delete=True)
            deploy = kwargs.get('deploy', None)
            if deploy:
                if (isinstance(deploy, ScriptDeployment) or
                        isinstance(deploy, SSHKeyDeployment)):
                    deployment = MultiStepDeployment([deploy,
                                                      root_enable_script])
                elif isinstance(deploy, MultiStepDeployment):
                    deployment = deploy
                    deployment.add(root_enable_script)
            else:
                deployment = root_enable_script

            kwargs['deploy'] = deployment

        if not kwargs.get('ssh_username', None):
            kwargs['ssh_username'] = 'toor'

        return super(ElasticStackBaseNodeDriver, self).deploy_node(**kwargs)

    def ex_shutdown_node(self, node):
        """
        Sends the ACPI power-down event

        :param      node: Node which should be used
        :type       node: :class:`Node`

        :rtype: ``bool``
        """
        response = self.connection.request(
            action='/servers/%s/shutdown' % (node.id),
            method='POST'
        )
        return response.status == 204

    def ex_destroy_drive(self, drive_uuid):
        """
        Deletes a drive

        :param      drive_uuid: Drive uuid which should be used
        :type       drive_uuid: ``str``

        :rtype: ``bool``
        """
        response = self.connection.request(
            action='/drives/%s/destroy' % (drive_uuid),
            method='POST'
        )
        return response.status == 204

    # Helper methods
    def _to_node(self, data, ssh_password=None):
        try:
            state = NODE_STATE_MAP[data['status']]
        except KeyError:
            state = NodeState.UNKNOWN

        if 'nic:0:dhcp:ip' in data:
            if isinstance(data['nic:0:dhcp:ip'], list):
                public_ip = data['nic:0:dhcp:ip']
            else:
                public_ip = [data['nic:0:dhcp:ip']]
        else:
            public_ip = []

        extra = {'cpu': data['cpu'],
                 'mem': data['mem']}

        if 'started' in data:
            extra['started'] = data['started']

        if 'smp' in data:
            extra['smp'] = data['smp']

        if 'vnc:ip' in data:
            extra['vnc:ip'] = data['vnc:ip']

        if 'vnc:password' in data:
            extra['vnc:password'] = data['vnc:password']

        boot_device = data['boot']

        if isinstance(boot_device, list):
            for device in boot_device:
                extra[device] = data[device]
        else:
            extra[boot_device] = data[boot_device]

        if ssh_password:
            extra.update({'password': ssh_password})

        node = Node(id=data['server'], name=data['name'], state=state,
                    public_ips=public_ip, private_ips=None,
                    driver=self.connection.driver,
                    extra=extra)

        return node



# Licensed to the Apache Software Foundation (ASF) under one or more
# contributor license agreements.  See the NOTICE file distributed with
# this work for additional information regarding copyright ownership.
# The ASF licenses this file to You under the Apache License, Version 2.0
# (the "License"); you may not use this file except in compliance with
# the License.  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""
Tests for Google Connection classes.
"""
import datetime
import mock
import os
import sys
import unittest

try:
    import simplejson as json
except ImportError:
    import json

from libcloud.common.google import (GoogleAuthError,
                                    GoogleAuthType,
                                    GoogleBaseAuthConnection,
                                    GoogleInstalledAppAuthConnection,
                                    GoogleServiceAcctAuthConnection,
                                    GoogleGCEServiceAcctAuthConnection,
                                    GoogleOAuth2Credential,
                                    GoogleBaseConnection,
                                    _utcnow,
                                    _utc_timestamp)
from libcloud.test import MockHttp, LibcloudTestCase
from libcloud.utils.py3 import httplib


# Skip some tests if PyCrypto is unavailable
try:
    from Crypto.Hash import SHA256
except ImportError:
    SHA256 = None


SCRIPT_PATH = os.path.dirname(os.path.realpath(__file__))
PEM_KEY = os.path.join(SCRIPT_PATH, 'fixtures', 'google', 'pkey.pem')
JSON_KEY = os.path.join(SCRIPT_PATH, 'fixtures', 'google', 'pkey.json')
with open(JSON_KEY, 'r') as f:
    KEY_STR = json.loads(f.read())['private_key']


GCE_PARAMS = ('email@developer.gserviceaccount.com', 'key')
GCE_PARAMS_PEM_KEY = ('email@developer.gserviceaccount.com', PEM_KEY)
GCE_PARAMS_JSON_KEY = ('email@developer.gserviceaccount.com', JSON_KEY)
GCE_PARAMS_KEY = ('email@developer.gserviceaccount.com', KEY_STR)
GCE_PARAMS_IA = ('client_id', 'client_secret')
GCE_PARAMS_GCE = ('foo', 'bar')
GCS_S3_PARAMS = ('GOOG0123456789ABCXYZ',  # GOOG + 16 alphanumeric chars
                 '0102030405060708091011121314151617181920')  # 40 base64 chars

STUB_UTCNOW = _utcnow()

STUB_TOKEN = {
    'access_token': 'tokentoken',
    'token_type': 'Bearer',
    'expires_in': 3600
}

STUB_IA_TOKEN = {
    'access_token': 'installedapp',
    'token_type': 'Bearer',
    'expires_in': 3600,
    'refresh_token': 'refreshrefresh'
}

STUB_REFRESH_TOKEN = {
    'access_token': 'refreshrefresh',
    'token_type': 'Bearer',
    'expires_in': 3600
}

STUB_TOKEN_FROM_FILE = {
    'access_token': 'token_from_file',
    'token_type': 'Bearer',
    'expire_time': _utc_timestamp(STUB_UTCNOW +
                                  datetime.timedelta(seconds=3600)),
    'expires_in': 3600
}


class MockJsonResponse(object):
    def __init__(self, body):
        self.object = body


class GoogleTestCase(LibcloudTestCase):
    """
    Assists in making Google tests hermetic and deterministic.

    Add anything that needs to be mocked here. Create a patcher with the
    suffix '_patcher'.

    e.g.
        _foo_patcher = mock.patch('module.submodule.class.foo', ...)

    Patchers are started at setUpClass and stopped at tearDownClass.

    Ideally, you should make a note in the thing being mocked, for clarity.
    """
    PATCHER_SUFFIX = '_patcher'

    _utcnow_patcher = mock.patch(
        'libcloud.common.google._utcnow', return_value=STUB_UTCNOW)

    _authtype_is_gce_patcher = mock.patch(
        'libcloud.common.google.GoogleAuthType._is_gce', return_value=False)

    _read_token_file_patcher = mock.patch(
        'libcloud.common.google.GoogleOAuth2Credential._get_token_from_file',
        return_value=STUB_TOKEN_FROM_FILE
    )

    _write_token_file_patcher = mock.patch(
        'libcloud.common.google.GoogleOAuth2Credential._write_token_to_file')

    _ia_get_code_patcher = mock.patch(
        'libcloud.common.google.GoogleInstalledAppAuthConnection.get_code',
        return_value=1234
    )

    @classmethod
    def setUpClass(cls):
        super(GoogleTestCase, cls).setUpClass()

        for patcher in [a for a in dir(cls) if a.endswith(cls.PATCHER_SUFFIX)]:
            getattr(cls, patcher).start()

    @classmethod
    def tearDownClass(cls):
        super(GoogleTestCase, cls).tearDownClass()

        for patcher in [a for a in dir(cls) if a.endswith(cls.PATCHER_SUFFIX)]:
            getattr(cls, patcher).stop()


class GoogleBaseAuthConnectionTest(GoogleTestCase):
    """
    Tests for GoogleBaseAuthConnection
    """

    def setUp(self):
        GoogleBaseAuthConnection.conn_classes = (GoogleAuthMockHttp,
                                                 GoogleAuthMockHttp)
        self.mock_scopes = ['foo', 'bar']
        kwargs = {'scopes': self.mock_scopes}
        self.conn = GoogleInstalledAppAuthConnection(*GCE_PARAMS,
                                                     **kwargs)

    def test_scopes(self):
        self.assertEqual(self.conn.scopes, 'foo bar')

    def test_add_default_headers(self):
        old_headers = {}
        expected_headers = {
            'Content-Type': 'application/x-www-form-urlencoded',
            'Host': 'accounts.google.com'}
        new_headers = self.conn.add_default_headers(old_headers)
        self.assertEqual(new_headers, expected_headers)

    def test_token_request(self):
        request_body = {'code': 'asdf', 'client_id': self.conn.user_id,
                        'client_secret': self.conn.key,
                        'redirect_uri': self.conn.redirect_uri,
                        'grant_type': 'authorization_code'}
        new_token = self.conn._token_request(request_body)
        self.assertEqual(new_token['access_token'],
                         STUB_IA_TOKEN['access_token'])
        exp = STUB_UTCNOW + datetime.timedelta(
            seconds=STUB_IA_TOKEN['expires_in'])
        self.assertEqual(new_token['expire_time'], _utc_timestamp(exp))


class GoogleInstalledAppAuthConnectionTest(GoogleTestCase):
    """
    Tests for GoogleInstalledAppAuthConnection
    """

    def setUp(self):
        GoogleInstalledAppAuthConnection.conn_classes = (GoogleAuthMockHttp,
                                                         GoogleAuthMockHttp)
        self.mock_scopes = ['https://www.googleapis.com/auth/foo']
        kwargs = {'scopes': self.mock_scopes}
        self.conn = GoogleInstalledAppAuthConnection(*GCE_PARAMS,
                                                     **kwargs)

    def test_refresh_token(self):
        # This token info doesn't have a refresh token, so a new token will be
        # requested
        token_info1 = {'access_token': 'tokentoken', 'token_type': 'Bearer',
                       'expires_in': 3600}
        new_token1 = self.conn.refresh_token(token_info1)
        self.assertEqual(new_token1['access_token'],
                         STUB_IA_TOKEN['access_token'])

        # This token info has a refresh token, so it will be able to be
        # refreshed.
        token_info2 = {'access_token': 'tokentoken', 'token_type': 'Bearer',
                       'expires_in': 3600, 'refresh_token': 'refreshrefresh'}
        new_token2 = self.conn.refresh_token(token_info2)
        self.assertEqual(new_token2['access_token'],
                         STUB_REFRESH_TOKEN['access_token'])

        # Both sets should have refresh info
        self.assertTrue('refresh_token' in new_token1)
        self.assertTrue('refresh_token' in new_token2)


class GoogleAuthTypeTest(GoogleTestCase):

    def test_guess(self):
        self.assertEqual(GoogleAuthType.guess_type(GCE_PARAMS[0]),
                         GoogleAuthType.SA)
        self.assertEqual(GoogleAuthType.guess_type(GCE_PARAMS_IA[0]),
                         GoogleAuthType.IA)
        with mock.patch.object(GoogleAuthType, '_is_gce', return_value=True):
            self.assertEqual(GoogleAuthType.guess_type(GCE_PARAMS_GCE[0]),
                             GoogleAuthType.GCE)
        self.assertEqual(
            GoogleAuthType.guess_type(GCS_S3_PARAMS[0]),
            GoogleAuthType.GCS_S3)


class GoogleOAuth2CredentialTest(GoogleTestCase):

    def test_init_oauth2(self):
        kwargs = {'auth_type': GoogleAuthType.IA}
        cred = GoogleOAuth2Credential(*GCE_PARAMS, **kwargs)

        # If there is a viable token file, this gets used first
        self.assertEqual(cred.token, STUB_TOKEN_FROM_FILE)

        # No token file, get a new token. Check that it gets written to file.
        with mock.patch.object(GoogleOAuth2Credential, '_get_token_from_file',
                               return_value=None):
            cred = GoogleOAuth2Credential(*GCE_PARAMS, **kwargs)
            expected = STUB_IA_TOKEN
            expected['expire_time'] = cred.token['expire_time']
            self.assertEqual(cred.token, expected)
            cred._write_token_to_file.assert_called_once_with()

    def test_refresh(self):
        args = list(GCE_PARAMS) + [GoogleAuthType.GCE]
        cred = GoogleOAuth2Credential(*args)
        cred._refresh_token = mock.Mock()

        # Test getting an unexpired access token.
        tomorrow = datetime.datetime.now() + datetime.timedelta(days=1)
        cred.token = {'access_token': 'Access Token!',
                      'expire_time': _utc_timestamp(tomorrow)}
        cred.access_token
        self.assertFalse(cred._refresh_token.called)

        # Test getting an expired access token.
        yesterday = datetime.datetime.now() - datetime.timedelta(days=1)
        cred.token = {'access_token': 'Access Token!',
                      'expire_time': _utc_timestamp(yesterday)}
        cred.access_token
        self.assertTrue(cred._refresh_token.called)

    def test_auth_connection(self):
        # Test a bogus auth type
        self.assertRaises(GoogleAuthError, GoogleOAuth2Credential, *GCE_PARAMS,
                          **{'auth_type': 'XX'})
        # Try to create an OAuth2 credential when dealing with a GCS S3
        # interoperability auth type.
        self.assertRaises(GoogleAuthError, GoogleOAuth2Credential, *GCE_PARAMS,
                          **{'auth_type': GoogleAuthType.GCS_S3})

        kwargs = {}

        if SHA256:
            kwargs['auth_type'] = GoogleAuthType.SA
            cred1 = GoogleOAuth2Credential(*GCE_PARAMS_PEM_KEY, **kwargs)
            self.assertTrue(isinstance(cred1.oauth2_conn,
                                       GoogleServiceAcctAuthConnection))

            cred1 = GoogleOAuth2Credential(*GCE_PARAMS_JSON_KEY, **kwargs)
            self.assertTrue(isinstance(cred1.oauth2_conn,
                                       GoogleServiceAcctAuthConnection))

            cred1 = GoogleOAuth2Credential(*GCE_PARAMS_KEY, **kwargs)
            self.assertTrue(isinstance(cred1.oauth2_conn,
                                       GoogleServiceAcctAuthConnection))

        kwargs['auth_type'] = GoogleAuthType.IA
        cred2 = GoogleOAuth2Credential(*GCE_PARAMS_IA, **kwargs)
        self.assertTrue(isinstance(cred2.oauth2_conn,
                                   GoogleInstalledAppAuthConnection))

        kwargs['auth_type'] = GoogleAuthType.GCE
        cred3 = GoogleOAuth2Credential(*GCE_PARAMS_GCE, **kwargs)
        self.assertTrue(isinstance(cred3.oauth2_conn,
                                   GoogleGCEServiceAcctAuthConnection))


class GoogleBaseConnectionTest(GoogleTestCase):
    """
    Tests for GoogleBaseConnection
    """

    def setUp(self):
        GoogleBaseAuthConnection.conn_classes = (GoogleAuthMockHttp,
                                                 GoogleAuthMockHttp)
        self.mock_scopes = ['https://www.googleapis.com/auth/foo']
        kwargs = {'scopes': self.mock_scopes,
                  'auth_type': GoogleAuthType.IA}
        self.conn = GoogleBaseConnection(*GCE_PARAMS, **kwargs)

    def test_add_default_headers(self):
        old_headers = {}
        new_expected_headers = {'Content-Type': 'application/json',
                                'Host': 'www.googleapis.com'}
        new_headers = self.conn.add_default_headers(old_headers)
        self.assertEqual(new_headers, new_expected_headers)

    def test_pre_connect_hook(self):
        old_params = {}
        old_headers = {}
        auth_str = '%s %s' % (STUB_TOKEN_FROM_FILE['token_type'],
                              STUB_TOKEN_FROM_FILE['access_token'])
        new_expected_params = {}
        new_expected_headers = {'Authorization': auth_str}
        new_params, new_headers = self.conn.pre_connect_hook(old_params,
                                                             old_headers)
        self.assertEqual(new_params, new_expected_params)
        self.assertEqual(new_headers, new_expected_headers)

    def test_encode_data(self):
        data = {'key': 'value'}
        json_data = '{"key": "value"}'
        encoded_data = self.conn.encode_data(data)
        self.assertEqual(encoded_data, json_data)

    def test_has_completed(self):
        body1 = {"endTime": "2013-06-26T10:05:07.630-07:00",
                 "id": "3681664092089171723",
                 "kind": "compute#operation",
                 "status": "DONE",
                 "targetId": "16211908079305042870"}
        body2 = {"endTime": "2013-06-26T10:05:07.630-07:00",
                 "id": "3681664092089171723",
                 "kind": "compute#operation",
                 "status": "RUNNING",
                 "targetId": "16211908079305042870"}
        response1 = MockJsonResponse(body1)
        response2 = MockJsonResponse(body2)
        self.assertTrue(self.conn.has_completed(response1))
        self.assertFalse(self.conn.has_completed(response2))

    def test_get_poll_request_kwargs(self):
        body = {"endTime": "2013-06-26T10:05:07.630-07:00",
                "id": "3681664092089171723",
                "kind": "compute#operation",
                "selfLink": "https://www.googleapis.com/operations-test"}
        response = MockJsonResponse(body)
        expected_kwargs = {'action':
                           'https://www.googleapis.com/operations-test'}
        kwargs = self.conn.get_poll_request_kwargs(response, None, {})
        self.assertEqual(kwargs, expected_kwargs)

    def test_morph_action_hook(self):
        self.conn.request_path = '/compute/apiver/project/project-name'
        action1 = ('https://www.googleapis.com/compute/apiver/project'
                   '/project-name/instances')
        action2 = '/instances'
        expected_request = '/compute/apiver/project/project-name/instances'
        request1 = self.conn.morph_action_hook(action1)
        request2 = self.conn.morph_action_hook(action2)
        self.assertEqual(request1, expected_request)
        self.assertEqual(request2, expected_request)


class GoogleAuthMockHttp(MockHttp):
    """
    Mock HTTP Class for Google Auth Connections.
    """
    json_hdr = {'content-type': 'application/json; charset=UTF-8'}

    def _o_oauth2_token(self, method, url, body, headers):
        if 'code' in body:
            body = json.dumps(STUB_IA_TOKEN)
        elif 'refresh_token' in body:
            body = json.dumps(STUB_REFRESH_TOKEN)
        else:
            body = json.dumps(STUB_TOKEN)
        return (httplib.OK, body, self.json_hdr, httplib.responses[httplib.OK])


if __name__ == '__main__':
    sys.exit(unittest.main())



# Licensed to the Apache Software Foundation (ASF) under one or more
# contributor license agreements.  See the NOTICE file distributed with
# this work for additional information regarding copyright ownership.
# The ASF licenses this file to You under the Apache License, Version 2.0
# (the "License"); you may not use this file except in compliance with
# the License.  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
HP Public cloud driver which is essentially just a small wrapper around
OpenStack driver.
"""

from libcloud.compute.types import Provider, LibcloudError
from libcloud.compute.drivers.openstack import OpenStack_1_1_Connection
from libcloud.compute.drivers.openstack import OpenStack_1_1_NodeDriver

__all__ = [
    'KiliCloudNodeDriver'
]

ENDPOINT_ARGS = {
    'service_type': 'compute',
    'name': 'nova',
    'region': 'RegionOne'
}

AUTH_URL = 'https://api.kili.io/keystone/v2.0/tokens'


class KiliCloudConnection(OpenStack_1_1_Connection):
    _auth_version = '2.0_password'

    def __init__(self, *args, **kwargs):
        self.region = kwargs.pop('region', None)
        self.get_endpoint_args = kwargs.pop('get_endpoint_args', None)
        super(KiliCloudConnection, self).__init__(*args, **kwargs)

    def get_endpoint(self):
        if not self.get_endpoint_args:
            raise LibcloudError(
                'KiliCloudConnection must have get_endpoint_args set')

        if '2.0_password' in self._auth_version:
            ep = self.service_catalog.get_endpoint(**self.get_endpoint_args)
        else:
            raise LibcloudError(
                'Auth version "%s" not supported' % (self._auth_version))

        public_url = ep.url

        if not public_url:
            raise LibcloudError('Could not find specified endpoint')

        return public_url


class KiliCloudNodeDriver(OpenStack_1_1_NodeDriver):
    name = 'Kili Public Cloud'
    website = 'http://kili.io/'
    connectionCls = KiliCloudConnection
    type = Provider.HPCLOUD

    def __init__(self, key, secret, tenant_name, secure=True,
                 host=None, port=None, **kwargs):
        """
        Note: tenant_name argument is required for Kili cloud.
        """
        self.tenant_name = tenant_name
        super(KiliCloudNodeDriver, self).__init__(key=key, secret=secret,
                                                  secure=secure, host=host,
                                                  port=port,
                                                  **kwargs)

    def _ex_connection_class_kwargs(self):
        kwargs = self.openstack_connection_kwargs()
        kwargs['get_endpoint_args'] = ENDPOINT_ARGS
        kwargs['ex_force_auth_url'] = AUTH_URL
        kwargs['ex_tenant_name'] = self.tenant_name

        return kwargs



# Licensed to the Apache Software Foundation (ASF) under one or more
# contributor license agreements.  See the NOTICE file distributed with
# this work for additional information regarding copyright ownership.
# The ASF licenses this file to You under the Apache License, Version 2.0
# (the "License"); you may not use this file except in compliance with
# the License.  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and

import sys
import unittest

from libcloud.utils.py3 import httplib

from libcloud.dns.types import ZoneDoesNotExistError
from libcloud.dns.types import RecordDoesNotExistError
from libcloud.dns.drivers.google import GoogleDNSDriver
from libcloud.common.google import GoogleBaseAuthConnection

from libcloud.test.common.test_google import GoogleAuthMockHttp, GoogleTestCase
from libcloud.test import MockHttpTestCase
from libcloud.test.file_fixtures import DNSFileFixtures
from libcloud.test.secrets import DNS_PARAMS_GOOGLE, DNS_KEYWORD_PARAMS_GOOGLE


class GoogleTests(GoogleTestCase):

    def setUp(self):
        GoogleDNSMockHttp.test = self
        GoogleDNSDriver.connectionCls.conn_classes = (GoogleDNSMockHttp,
                                                      GoogleDNSMockHttp)
        GoogleBaseAuthConnection.conn_classes = (GoogleAuthMockHttp,
                                                 GoogleAuthMockHttp)
        GoogleDNSMockHttp.type = None
        kwargs = DNS_KEYWORD_PARAMS_GOOGLE.copy()
        kwargs['auth_type'] = 'IA'
        self.driver = GoogleDNSDriver(*DNS_PARAMS_GOOGLE, **kwargs)

    def test_default_scopes(self):
        self.assertEqual(self.driver.scopes, None)

    def test_list_zones(self):
        zones = self.driver.list_zones()
        self.assertEqual(len(zones), 2)

    def test_list_records(self):
        zone = self.driver.list_zones()[0]
        records = self.driver.list_records(zone=zone)
        self.assertEqual(len(records), 3)

    def test_get_zone(self):
        zone = self.driver.get_zone('example-com')
        self.assertEqual(zone.id, 'example-com')
        self.assertEqual(zone.domain, 'example.com.')

    def test_get_zone_does_not_exist(self):
        GoogleDNSMockHttp.type = 'ZONE_DOES_NOT_EXIST'

        try:
            self.driver.get_zone('example-com')
        except ZoneDoesNotExistError:
            e = sys.exc_info()[1]
            self.assertEqual(e.zone_id, 'example-com')
        else:
            self.fail('Exception not thrown')

    def test_get_record(self):
        GoogleDNSMockHttp.type = 'FILTER_ZONES'
        zone = self.driver.list_zones()[0]
        record = self.driver.get_record(zone.id, "A:foo.example.com.")
        self.assertEqual(record.id, 'A:foo.example.com.')
        self.assertEqual(record.name, 'foo.example.com.')
        self.assertEqual(record.type, 'A')
        self.assertEqual(record.zone.id, 'example-com')

    def test_get_record_zone_does_not_exist(self):
        GoogleDNSMockHttp.type = 'ZONE_DOES_NOT_EXIST'

        try:
            self.driver.get_record('example-com', 'a:a')
        except ZoneDoesNotExistError:
            e = sys.exc_info()[1]
            self.assertEqual(e.zone_id, 'example-com')
        else:
            self.fail('Exception not thrown')

    def test_get_record_record_does_not_exist(self):
        GoogleDNSMockHttp.type = 'RECORD_DOES_NOT_EXIST'
        try:
            self.driver.get_record('example-com', "A:foo")
        except RecordDoesNotExistError:
            e = sys.exc_info()[1]
            self.assertEqual(e.record_id, 'A:foo')
        else:
            self.fail('Exception not thrown')

    def test_create_zone(self):
        extra = {'description': 'new domain for example.org'}
        zone = self.driver.create_zone('example.org.', extra)
        self.assertEqual(zone.domain, 'example.org.')
        self.assertEqual(zone.extra['description'], extra['description'])
        self.assertEqual(len(zone.extra['nameServers']), 4)

    def test_delete_zone(self):
        zone = self.driver.get_zone('example-com')
        res = self.driver.delete_zone(zone)
        self.assertTrue(res)

    def test_ex_bulk_record_changes(self):
        zone = self.driver.get_zone('example-com')
        records = self.driver.ex_bulk_record_changes(zone, {})

        self.assertEqual(records['additions'][0].name, 'foo.example.com.')
        self.assertEqual(records['additions'][0].type, 'A')

        self.assertEqual(records['deletions'][0].name, 'bar.example.com.')
        self.assertEqual(records['deletions'][0].type, 'A')


class GoogleDNSMockHttp(MockHttpTestCase):
    fixtures = DNSFileFixtures('google')

    def _dns_v1_projects_project_name_managedZones(
            self, method, url, body, headers):
        if method == 'POST':
            body = self.fixtures.load('zone_create.json')
        else:
            body = self.fixtures.load('zone_list.json')
        return (httplib.OK, body, {}, httplib.responses[httplib.OK])

    def _dns_v1_projects_project_name_managedZones_FILTER_ZONES(
            self, method, url, body, headers):
        body = self.fixtures.load('zone_list.json')
        return (httplib.OK, body, {}, httplib.responses[httplib.OK])

    def _dns_v1_projects_project_name_managedZones_example_com_rrsets_FILTER_ZONES(
            self, method, url, body, headers):
        body = self.fixtures.load('record.json')
        return (httplib.OK, body, {}, httplib.responses[httplib.OK])

    def _dns_v1_projects_project_name_managedZones_example_com_rrsets(
            self, method, url, body, headers):
        body = self.fixtures.load('records_list.json')
        return (httplib.OK, body, {}, httplib.responses[httplib.OK])

    def _dns_v1_projects_project_name_managedZones_example_com(
            self, method, url, body, headers):
        if method == 'GET':
            body = self.fixtures.load('managed_zones_1.json')
        elif method == 'DELETE':
            body = None
        return (httplib.OK, body, {}, httplib.responses[httplib.OK])

    def _dns_v1_projects_project_name_managedZones_example_com_changes(
            self, method, url, body, headers):
        body = self.fixtures.load('record_changes.json')
        return (httplib.OK, body, {}, httplib.responses[httplib.OK])

    def _dns_v1_projects_project_name_managedZones_example_com_ZONE_DOES_NOT_EXIST(
            self, method, url, body, headers):
        body = self.fixtures.load('get_zone_does_not_exists.json')
        return (httplib.NOT_FOUND, body, {},
                httplib.responses[httplib.NOT_FOUND])

    def _dns_v1_projects_project_name_managedZones_example_com_RECORD_DOES_NOT_EXIST(
            self, method, url, body, headers):
        body = self.fixtures.load('managed_zones_1.json')
        return (httplib.OK, body, {}, httplib.responses[httplib.OK])

    def _dns_v1_projects_project_name_managedZones_example_com_rrsets_RECORD_DOES_NOT_EXIST(
            self, method, url, body, headers):
        body = self.fixtures.load('no_record.json')
        return (httplib.OK, body, {}, httplib.responses[httplib.OK])

    def _dns_v1_projects_project_name_managedZones_example_com_rrsets_ZONE_DOES_NOT_EXIST(
            self, method, url, body, headers):
        body = self.fixtures.load('get_zone_does_not_exists.json')
        return (httplib.NOT_FOUND, body, {},
                httplib.responses[httplib.NOT_FOUND])

    def _dns_v1_projects_project_name_managedZones_example_com_FILTER_ZONES(
            self, method, url, body, headers):
        body = self.fixtures.load('zone.json')
        return (httplib.OK, body, {}, httplib.responses[httplib.OK])

if __name__ == '__main__':
    sys.exit(unittest.main())



# Licensed to the Apache Software Foundation (ASF) under one or more
# contributor license agreements.  See the NOTICE file distributed with
# this work for additional information regarding copyright ownership.
# The ASF licenses this file to You under the Apache License, Version 2.0
# (the "License"); you may not use this file except in compliance with
# the License.  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import with_statement

import os
import sys
import shutil
import unittest
import tempfile

import mock

from libcloud.common.types import LibcloudError
from libcloud.storage.base import Container
from libcloud.storage.types import ContainerDoesNotExistError
from libcloud.storage.types import ContainerAlreadyExistsError
from libcloud.storage.types import ContainerIsNotEmptyError
from libcloud.storage.types import InvalidContainerNameError

try:
    from libcloud.storage.drivers.local import LocalStorageDriver
    from libcloud.storage.drivers.local import LockLocalStorage
    from lockfile import LockTimeout
except ImportError:
    print('lockfile library is not available, skipping local_storage tests...')
    LocalStorageDriver = None
    LockTimeout = None


class LocalTests(unittest.TestCase):
    driver_type = LocalStorageDriver

    @classmethod
    def create_driver(self):
        self.key = tempfile.mkdtemp()
        return self.driver_type(self.key, None)

    def setUp(self):
        self.driver = self.create_driver()

    def tearDown(self):
        shutil.rmtree(self.key)
        self.key = None

    def make_tmp_file(self):
        _, tmppath = tempfile.mkstemp()
        with open(tmppath, 'wb') as fp:
            fp.write(b'blah' * 1024)
        return tmppath

    def remove_tmp_file(self, tmppath):
        os.unlink(tmppath)

    def test_list_containers_empty(self):
        containers = self.driver.list_containers()
        self.assertEqual(len(containers), 0)

    def test_containers_success(self):
        self.driver.create_container('test1')
        self.driver.create_container('test2')
        containers = self.driver.list_containers()
        self.assertEqual(len(containers), 2)

        container = containers[1]

        self.assertTrue('creation_time' in container.extra)
        self.assertTrue('modify_time' in container.extra)
        self.assertTrue('access_time' in container.extra)

        objects = self.driver.list_container_objects(container=container)
        self.assertEqual(len(objects), 0)

        objects = container.list_objects()
        self.assertEqual(len(objects), 0)

        for container in containers:
            self.driver.delete_container(container)

    def test_objects_success(self):
        tmppath = self.make_tmp_file()

        container = self.driver.create_container('test3')
        obj1 = container.upload_object(tmppath, 'object1')
        obj2 = container.upload_object(tmppath, 'path/object2')
        obj3 = container.upload_object(tmppath, 'path/to/object3')
        obj4 = container.upload_object(tmppath, 'path/to/object4.ext')
        with open(tmppath, 'rb') as tmpfile:
            obj5 = container.upload_object_via_stream(tmpfile, 'object5')

        objects = self.driver.list_container_objects(container=container)
        self.assertEqual(len(objects), 5)

        for obj in objects:
            self.assertNotEqual(obj.hash, None)
            self.assertEqual(obj.size, 4096)
            self.assertEqual(obj.container.name, 'test3')
            self.assertTrue('creation_time' in obj.extra)
            self.assertTrue('modify_time' in obj.extra)
            self.assertTrue('access_time' in obj.extra)

        obj1.delete()
        obj2.delete()

        objects = container.list_objects()
        self.assertEqual(len(objects), 3)

        container.delete_object(obj3)
        container.delete_object(obj4)
        container.delete_object(obj5)

        objects = container.list_objects()
        self.assertEqual(len(objects), 0)

        container.delete()
        self.remove_tmp_file(tmppath)

    def test_get_container_doesnt_exist(self):
        try:
            self.driver.get_container(container_name='container1')
        except ContainerDoesNotExistError:
            pass
        else:
            self.fail('Exception was not thrown')

    def test_get_container_success(self):
        self.driver.create_container('test4')
        container = self.driver.get_container(container_name='test4')
        self.assertTrue(container.name, 'test4')
        container.delete()

    def test_get_object_container_doesnt_exist(self):
        try:
            self.driver.get_object(container_name='test-inexistent',
                                   object_name='test')
        except ContainerDoesNotExistError:
            pass
        else:
            self.fail('Exception was not thrown')

    def test_get_object_success(self):
        tmppath = self.make_tmp_file()
        container = self.driver.create_container('test5')
        container.upload_object(tmppath, 'test')

        obj = self.driver.get_object(container_name='test5',
                                     object_name='test')

        self.assertEqual(obj.name, 'test')
        self.assertEqual(obj.container.name, 'test5')
        self.assertEqual(obj.size, 4096)
        self.assertNotEqual(obj.hash, None)
        self.assertTrue('creation_time' in obj.extra)
        self.assertTrue('modify_time' in obj.extra)
        self.assertTrue('access_time' in obj.extra)

        obj.delete()
        container.delete()
        self.remove_tmp_file(tmppath)

    def test_create_container_invalid_name(self):
        try:
            self.driver.create_container(container_name='new/container')
        except InvalidContainerNameError:
            pass
        else:
            self.fail('Exception was not thrown')

    def test_create_container_already_exists(self):
        container = self.driver.create_container(
            container_name='new-container')
        try:
            self.driver.create_container(container_name='new-container')
        except ContainerAlreadyExistsError:
            pass
        else:
            self.fail('Exception was not thrown')

        # success
        self.driver.delete_container(container)

    def test_create_container_success(self):
        name = 'new_container'
        container = self.driver.create_container(container_name=name)
        self.assertEqual(container.name, name)
        self.driver.delete_container(container)

    def test_delete_container_doesnt_exist(self):
        container = Container(name='new_container', extra=None,
                              driver=self.driver)
        try:
            self.driver.delete_container(container=container)
        except ContainerDoesNotExistError:
            pass
        else:
            self.fail('Exception was not thrown')

    def test_delete_container_not_empty(self):
        tmppath = self.make_tmp_file()
        container = self.driver.create_container('test6')
        obj = container.upload_object(tmppath, 'test')

        try:
            self.driver.delete_container(container=container)
        except ContainerIsNotEmptyError:
            pass
        else:
            self.fail('Exception was not thrown')

        # success
        obj.delete()
        self.remove_tmp_file(tmppath)
        self.assertTrue(self.driver.delete_container(container=container))

    def test_delete_container_not_found(self):
        container = Container(name='foo_bar_container', extra={},
                              driver=self.driver)
        try:
            self.driver.delete_container(container=container)
        except ContainerDoesNotExistError:
            pass
        else:
            self.fail('Container does not exist but an exception was not' +
                      'thrown')

    def test_delete_container_success(self):
        container = self.driver.create_container('test7')
        self.assertTrue(self.driver.delete_container(container=container))

    def test_download_object_success(self):
        tmppath = self.make_tmp_file()
        container = self.driver.create_container('test6')
        obj = container.upload_object(tmppath, 'test')

        destination_path = tmppath + '.temp'
        result = self.driver.download_object(obj=obj,
                                             destination_path=destination_path,
                                             overwrite_existing=False,
                                             delete_on_failure=True)

        self.assertTrue(result)

        obj.delete()
        container.delete()
        self.remove_tmp_file(tmppath)
        os.unlink(destination_path)

    def test_download_object_and_overwrite(self):
        tmppath = self.make_tmp_file()
        container = self.driver.create_container('test6')
        obj = container.upload_object(tmppath, 'test')

        destination_path = tmppath + '.temp'
        result = self.driver.download_object(obj=obj,
                                             destination_path=destination_path,
                                             overwrite_existing=False,
                                             delete_on_failure=True)

        self.assertTrue(result)

        try:
            self.driver.download_object(obj=obj,
                                        destination_path=destination_path,
                                        overwrite_existing=False,
                                        delete_on_failure=True)
        except LibcloudError:
            pass
        else:
            self.fail('Exception was not thrown')

        result = self.driver.download_object(obj=obj,
                                             destination_path=destination_path,
                                             overwrite_existing=True,
                                             delete_on_failure=True)

        self.assertTrue(result)

        # success
        obj.delete()
        container.delete()
        self.remove_tmp_file(tmppath)
        os.unlink(destination_path)

    def test_download_object_as_stream_success(self):
        tmppath = self.make_tmp_file()
        container = self.driver.create_container('test6')
        obj = container.upload_object(tmppath, 'test')

        stream = self.driver.download_object_as_stream(obj=obj,
                                                       chunk_size=1024)

        self.assertTrue(hasattr(stream, '__iter__'))

        data = b''.join(stream)
        self.assertTrue(len(data), 4096)

        obj.delete()
        container.delete()
        self.remove_tmp_file(tmppath)

    @mock.patch("lockfile.mkdirlockfile.MkdirLockFile.acquire",
                mock.MagicMock(side_effect=LockTimeout))
    def test_proper_lockfile_imports(self):
        # LockLocalStorage was previously using an un-imported exception
        # in its __enter__ method, so the following would raise a NameError.
        lls = LockLocalStorage("blah")
        self.assertRaises(LibcloudError, lls.__enter__)


if not LocalStorageDriver:
    class LocalTests(unittest.TestCase):  # NOQA
        pass


if __name__ == '__main__':
    sys.exit(unittest.main())



# Licensed to the Apache Software Foundation (ASF) under one or more
# contributor license agreements.  See the NOTICE file distributed with
# this work for additional information regarding copyright ownership.
# The ASF licenses this file to You under the Apache License, Version 2.0
# (the "License"); you may not use this file except in compliance with
# the License.  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Provides generic deployment steps for machines post boot.
"""

from __future__ import with_statement

import os
import binascii

from libcloud.utils.py3 import basestring, PY3


class Deployment(object):
    """
    Base class for deployment tasks.
    """

    def run(self, node, client):
        """
        Runs this deployment task on node using the client provided.

        :type node: :class:`Node`
        :keyword node: Node to operate one

        :type client: :class:`BaseSSHClient`
        :keyword client: Connected SSH client to use.

        :return: :class:`Node`
        """
        raise NotImplementedError(
            'run not implemented for this deployment')

    def _get_string_value(self, argument_name, argument_value):
        if not isinstance(argument_value, basestring) and \
           not hasattr(argument_value, 'read'):
            raise TypeError('%s argument must be a string or a file-like '
                            'object' % (argument_name))

        if hasattr(argument_value, 'read'):
            argument_value = argument_value.read()

        return argument_value


class SSHKeyDeployment(Deployment):
    """
    Installs a public SSH Key onto a server.
    """

    def __init__(self, key):
        """
        :type key: ``str`` or :class:`File` object
        :keyword key: Contents of the public key write or a file object which
                      can be read.
        """
        self.key = self._get_string_value(argument_name='key',
                                          argument_value=key)

    def run(self, node, client):
        """
        Installs SSH key into ``.ssh/authorized_keys``

        See also :class:`Deployment.run`
        """
        client.put(".ssh/authorized_keys", contents=self.key, mode='a')
        return node


class FileDeployment(Deployment):
    """
    Installs a file on the server.
    """

    def __init__(self, source, target):
        """
        :type source: ``str``
        :keyword source: Local path of file to be installed

        :type target: ``str``
        :keyword target: Path to install file on node
        """
        self.source = source
        self.target = target

    def run(self, node, client):
        """
        Upload the file, retaining permissions.

        See also :class:`Deployment.run`
        """
        perms = int(oct(os.stat(self.source).st_mode)[4:], 8)

        with open(self.source, 'rb') as fp:
            content = fp.read()

        client.put(path=self.target, chmod=perms,
                   contents=content)
        return node


class ScriptDeployment(Deployment):
    """
    Runs an arbitrary shell script on the server.

    This step works by first writing the content of the shell script (script
    argument) in a \*.sh file on a remote server and then running that file.

    If you are running a non-shell script, make sure to put the appropriate
    shebang to the top of the script. You are also advised to do that even if
    you are running a plan shell script.
    """

    def __init__(self, script, args=None, name=None, delete=False):
        """
        :type script: ``str``
        :keyword script: Contents of the script to run.

        :type args: ``list``
        :keyword args: Optional command line arguments which get passed to the
                       deployment script file.

        :type name: ``str``
        :keyword name: Name of the script to upload it as, if not specified,
                       a random name will be chosen.

        :type delete: ``bool``
        :keyword delete: Whether to delete the script on completion.
        """
        script = self._get_string_value(argument_name='script',
                                        argument_value=script)

        self.script = script
        self.args = args or []
        self.stdout = None
        self.stderr = None
        self.exit_status = None
        self.delete = delete
        self.name = name

        if self.name is None:
            # File is put under user's home directory
            # (~/libcloud_deployment_<random_string>.sh)
            random_string = binascii.hexlify(os.urandom(4))
            random_string = random_string.decode('ascii')
            self.name = 'libcloud_deployment_%s.sh' % (random_string)

    def run(self, node, client):
        """
        Uploads the shell script and then executes it.

        See also :class:`Deployment.run`
        """
        file_path = client.put(path=self.name, chmod=int('755', 8),
                               contents=self.script)

        # Pre-pend cwd if user specified a relative path
        if self.name[0] != '/':
            base_path = os.path.dirname(file_path)
            name = os.path.join(base_path, self.name)
        else:
            name = self.name

        cmd = name

        if self.args:
            # Append arguments to the command
            cmd = '%s %s' % (name, ' '.join(self.args))
        else:
            cmd = name

        self.stdout, self.stderr, self.exit_status = client.run(cmd)

        if self.delete:
            client.delete(self.name)

        return node


class ScriptFileDeployment(ScriptDeployment):
    """
    Runs an arbitrary shell script from a local file on the server. Same as
    ScriptDeployment, except that you can pass in a path to the file instead of
    the script content.
    """

    def __init__(self, script_file, args=None, name=None, delete=False):
        """
        :type script_file: ``str``
        :keyword script_file: Path to a file containing the script to run.

        :type args: ``list``
        :keyword args: Optional command line arguments which get passed to the
                       deployment script file.


        :type name: ``str``
        :keyword name: Name of the script to upload it as, if not specified,
                       a random name will be chosen.

        :type delete: ``bool``
        :keyword delete: Whether to delete the script on completion.
        """
        with open(script_file, 'rb') as fp:
            content = fp.read()

        if PY3:
            content = content.decode('utf-8')

        super(ScriptFileDeployment, self).__init__(script=content,
                                                   args=args,
                                                   name=name,
                                                   delete=delete)


class MultiStepDeployment(Deployment):
    """
    Runs a chain of Deployment steps.
    """
    def __init__(self, add=None):
        """
        :type add: ``list``
        :keyword add: Deployment steps to add.
        """
        self.steps = []
        self.add(add)

    def add(self, add):
        """
        Add a deployment to this chain.

        :type add: Single :class:`Deployment` or a ``list`` of
                   :class:`Deployment`
        :keyword add: Adds this deployment to the others already in this
                      object.
        """
        if add is not None:
            add = add if isinstance(add, (list, tuple)) else [add]
            self.steps.extend(add)

    def run(self, node, client):
        """
        Run each deployment that has been added.

        See also :class:`Deployment.run`
        """
        for s in self.steps:
            node = s.run(node, client)
        return node
